/*
 * Copyright Freescale Semiconductor, Inc.
 * Copyright (C) 2007 Sony Corporation
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 * Change log:
 * 20050101: Eran Liberty (liberty@freescale.com)
 *           Initial file creating (porting from 85XX & 8260)
 * 20071020: AKA <masateru.akachi@jp.sony.com>
 *           Initial file creating (porting from mpc8349ads.c)
 * 20080526: AKA <masateru.akachi@jp.sony.com>
 *           adjusted for Mass Production branch
 */

#include <common.h>
#include <ioports.h>
#include <mpc83xx.h>
#include <asm/mpc8349_pci.h>
#include <i2c.h>
#include <spd.h>
#include <miiphy.h>
#if defined(CONFIG_PCI)
#include <pci.h>
#endif
#if defined(CONFIG_SPD_EEPROM)
#include <spd_sdram.h>
#endif
#include	"Bellini_local.h"

int fixed_sdram(void);
void sdram_init(void);

#if defined( CONFIG_SETUP_UPM )
	extern void	setupUPM( void );
#endif	/* CONFIG_SETUP_UPM */


int board_early_init_f (void)
/*
 * This function called BEFORE preparing console, environment variables, or so, thus, any message can NOT be displayed here.
 * See init_sequence[] definition in libppc/board.c for details.
 */
{
	volatile immap_t *im = (immap_t *)CFG_IMMRBAR;
#if defined( CONFIG_PATCH_ERRATA_TSEC_6_1 )
	volatile TSEC_RX_FIFO_t	*tsec1_rx_fifo = (TSEC_RX_FIFO_t *)CFG_TSEC1;
	volatile TSEC_RX_FIFO_t	*tsec2_rx_fifo = (TSEC_RX_FIFO_t *)CFG_TSEC2;

	tsec1_rx_fifo->alarm = 0x080;
	tsec1_rx_fifo->alarm_shutoff = 0x040;
	tsec1_rx_fifo->panic = 0x100;
	tsec1_rx_fifo->panic_shutoff = 0x080;
	tsec2_rx_fifo->alarm = 0x080;
	tsec2_rx_fifo->alarm_shutoff = 0x040;
	tsec2_rx_fifo->panic = 0x100;
	tsec2_rx_fifo->panic_shutoff = 0x080;
#endif	/* CONFIG_PATCH_ERRATA_TSEC_6_1 */

	im->i2c[0].fdr = CFG_I2C_FDR;
	im->i2c[1].fdr = CFG_I2C2_FDR;

	return 0;
}


int	misc_init_r (void)
/* This function is called AFTER console_init_r() running on RAM.
 * Now, console device and accessing main storage (RAM) are available here.
 * See board_init_r() in libppc/board.c for details.
 * Prototype is defined in common.h. It assumes int func(void), however the return value is ignored by caller(board_init_f_r()).
 */
{
	volatile immap_t	*im = (immap_t *)CFG_IMMRBAR;
	eepromflags_t	eeprom_flags = {};	/* filled all with 0 */
	int				result;

	/* Because OCCR is overwritten in cpu/mpc83xx/pci.c after board_early_init_f(), we have to overwrite again... */
	im->clk.occr = BITOP_CLEAR( im->clk.occr, CFG_OCCR_DISABLE_MASK );

	result = get_eeprom_flag( &eeprom_flags );
	
	set_vfreqmode( &eeprom_flags );
	overwrite_env( &eeprom_flags );

	return result;
}

#define	SIZ_I2C_BUF		sizeof( uint8_t )	/* enough for each field size of bootmode, diagpass, freqmode IN EEP-ROM */
/* --- */
static int	get_eeprom_flag( eepromflags_t *eeprom_flags )
/* store to eepromflags_t by reading EEP-ROM: 
 *	(in) *eeprom_flags: pointer of the object assumed all members are 0 initialized.
 *  (out) error code: 0=No error
*/
{
	const CFG_EEPROM_ID_t	*const dummy_ptr_id = NULL;	/* only used for examine sizeof_member */
	const size_t			sizeof_mac0 = sizeof(dummy_ptr_id->mac_add_1);
	const I2C_ADDR_t		rom_ofs_mac0 = CFG_EEPROM_ID_BASE + offsetof( CFG_EEPROM_ID_t, mac_add_1 );
	const size_t			sizeof_mac1 = sizeof(dummy_ptr_id->mac_add_2);
	const I2C_ADDR_t		rom_ofs_mac1 = CFG_EEPROM_ID_BASE + offsetof( CFG_EEPROM_ID_t, mac_add_2 );

	const CFG_EEPROM_BOOT_t	*const dummy_ptr_boot = NULL;
	const size_t			sizeof_bootmode = sizeof(dummy_ptr_boot->boot_mode);
	const I2C_ADDR_t		rom_ofs_bootmode = CFG_EEPROM_BOOT_BASE + offsetof( CFG_EEPROM_BOOT_t, boot_mode );
	const size_t			sizeof_vfreqmode = sizeof(dummy_ptr_boot->frequency_mode);
	const I2C_ADDR_t		rom_ofs_vfreqmode = CFG_EEPROM_BOOT_BASE + offsetof( CFG_EEPROM_BOOT_t, frequency_mode );

	const CFG_EEPROM_DIAG_t	*const dummy_ptr_diag = NULL;
	const size_t			sizeof_diag_pass = sizeof(dummy_ptr_diag->diag_pass);
	const I2C_ADDR_t		rom_ofs_diag_pass = CFG_EEPROM_DIAG_BASE + offsetof( CFG_EEPROM_DIAG_t, diag_pass );

	uint8_t					i2c_buf[ SIZ_I2C_BUF ];
	int						result = 0;

	result = i2c_read( CFG_I2C_EEPROM_ADDR, rom_ofs_mac0, CFG_I2C_EEPROM_ADDR_LEN
						, eeprom_flags->mac0_addr, sizeof_mac0 );
	EXCEPTION_IF( result != 0, excep_get_eeprom_flag );
	result = i2c_read( CFG_I2C_EEPROM_ADDR, rom_ofs_mac1, CFG_I2C_EEPROM_ADDR_LEN
						, eeprom_flags->mac1_addr, sizeof_mac1 );
	EXCEPTION_IF( result != 0, excep_get_eeprom_flag );

	result = i2c_read( CFG_I2C_EEPROM_ADDR, rom_ofs_bootmode, CFG_I2C_EEPROM_ADDR_LEN, i2c_buf, sizeof_bootmode );
	EXCEPTION_IF( result != 0, excep_get_eeprom_flag );
	eeprom_flags->boot_sel = (int)( (unsigned int)(*i2c_buf) );
	result = i2c_read( CFG_I2C_EEPROM_ADDR, rom_ofs_vfreqmode, CFG_I2C_EEPROM_ADDR_LEN, i2c_buf, sizeof_vfreqmode );
	EXCEPTION_IF( result != 0, excep_get_eeprom_flag );
	eeprom_flags->vfreq_mode = (int)( (unsigned int)(*i2c_buf) );

	result = i2c_read( CFG_I2C_EEPROM_ADDR, rom_ofs_diag_pass, CFG_I2C_EEPROM_ADDR_LEN, i2c_buf, sizeof_diag_pass );
	EXCEPTION_IF( result != 0, excep_get_eeprom_flag );
	eeprom_flags->f_diag_pass = (int)( (unsigned int)(*i2c_buf) );

excep_get_eeprom_flag:
	return result;
}
#undef	SIZ_I2C_BUF

/* --- */
static void	set_vfreqmode( const eepromflags_t *eeprom_flags )
/* set video frequency by EEP-ROM */
{
	uint8_t	*const modereg = (uint8_t *)CFG_BELLINI_ADDR_FREQMODE_REG;

	*modereg = ( eeprom_flags->vfreq_mode == BEL_DISPFREQ_50HZ )
					? CFG_BELLINI_DATA_FREQMODE_50HZ : CFG_BELLINI_DATA_FREQMODE_60HZ;
	return;
}

/* --- */
static void	overwrite_env( const eepromflags_t *eeprom_flags )
{
	setenv_uboot_parameters( );
	setenv_scripts( );

	setenv_by_eeprom( eeprom_flags );
	return;
}

/* --- */
static void	setenv_uboot_parameters( void )
{
	char	buf[ CFG_BELLINI_INIT_R_WORKBUF_SIZE ];

	sprintf( buf, "%X", TEXT_BASE );
	setenv( CFG_ENV_MONITOR_SADDR_NAME, buf );
	sprintf( buf, "%X", CFG_ENV_ADDR );
	setenv( CFG_ENV_ENV_SADDR_NAME, buf );
	sprintf( buf, "%X", CFG_ENV_SIZE );
	setenv( CFG_ENV_ENV_SIZE_NAME, buf );

	sprintf( buf, "%X", CFG_RCW_ADDD );
	setenv( CFG_ENV_RCW_ADDR_NAME, buf );
	sprintf( buf, "%X", CFG_RCW_SIZE );
	setenv( CFG_ENV_RCW_SIZE_NAME, buf );

	{
		ulong	eovec;	/* Address of EndOfVector */
		ulong	sp;		/* Address of StackPointer */
		extern ulong  _start;			/* member of GOT */
		extern ulong  _end_of_vectors;

		eovec = (ulong)(& _end_of_vectors) - ( (ulong)(& _start) - DEF_MEMTEST_VECTOR_RESET ) + DEF_MEMTEST_VECTOR_START;
		sprintf( buf, "%X", eovec + CFG_MEMTEST_VECTOR_GUARD_SIZE );
		setenv( CFG_ENV_MTEST_START_NAME, buf );

		asm( "mr %0, 1": "=r"(sp) : );	/* sp<=R1(StackPointer) */
		sprintf( buf, "%X", sp - CFG_MEMTEST_STACK_GUARD_SIZE );
		setenv( CFG_ENV_MTEST_END_NAME, buf );
	}
	return;
}

/* --- */
static void	setenv_scripts( void )
{
	setenv( CFG_ENV_LOADADDR_DFL_NAME, CFG_ENV_LOADADDR_DFL_COMMAND( CONFIG_LOADADDR ) );
	setenv( CFG_ENV_NORIMAGE_ADDR_NAME, CFG_ENV_NORIMAGE_ADDR );
	setenv( CFG_ENV_NORINITRD_ADDR_NAME, CFG_ENV_NORINITRD_ADDR );

	setenv( "autoload", "no" );
	DEFENV( CFG_ENV_AUTOSETUP_CMD_NAME, CFG_ENV_AUTOSETUP_COMMAND );

	setenv( CFG_ENV_BOOT0_CMD_NAME, CFG_ENV_BOOT0_COMMAND );
	DEFENV( CFG_ENV_BOOT1_CMD_NAME, CFG_ENV_BOOT1_COMMAND );
	DEFENV( CFG_ENV_BOOT2_CMD_NAME, CFG_ENV_BOOT2_COMMAND );
	DEFENV( CFG_ENV_BOOT3_CMD_NAME, CFG_ENV_BOOT3_COMMAND );
	DEFENV( CFG_ENV_BOOT4_CMD_NAME, CFG_ENV_BOOT4_COMMAND );

	setenv( CFG_ENV_SETFTRANS_NONE_CMD_NAME, CFG_ENV_SETFTRANS_NONE_COMMAND );
	setenv( CFG_ENV_SETFTRANS_ATA_CMD_NAME,  CFG_ENV_SETFTRANS_ATA_COMMAND );
	setenv( CFG_ENV_SETFTRANS_TFTP_CMD_NAME, CFG_ENV_SETFTRANS_TFTP_COMMAND );
	setenv( CFG_ENV_SETFTRANS_NFS_CMD_NAME,  CFG_ENV_SETFTRANS_NFS_COMMAND );
	setenv( CFG_ENV_BOOTSTRAP_CMD_NAME, CFG_ENV_BOOTSTRAP_COMMAND );
	setenv( CFG_ENV_BOOT_HOOK_CMD_NAME, CFG_ENV_BOOT_HOOK_COMMAND );

	setenv( CFG_ENV_ERASE_ENV_CMD_NAME, CFG_ENV_ERASE_ENV_COMMAND );

	setenv( CFG_ENV_Dhcp_CMD_NAME, CFG_ENV_Dhcp_COMMAND );

	DEFENV( CFG_ENV_NFSBOOT_Dhcp_CMD_NAME, CFG_ENV_NFSBOOT_Dhcp_COMMAND );
	DEFENV( CFG_ENV_NFSBOOT_CMD_NAME, CFG_ENV_NFSBOOT_COMMAND );
	DEFENV( CFG_ENV_NFSBOOT_TFTP_Dhcp_CMD_NAME, CFG_ENV_NFSBOOT_TFTP_Dhcp_COMMAND );
	DEFENV( CFG_ENV_NFSBOOT_TFTP_CMD_NAME, CFG_ENV_NFSBOOT_TFTP_COMMAND );
	DEFENV( CFG_ENV_NFSBOOT_NFS_Dhcp_CMD_NAME, CFG_ENV_NFSBOOT_NFS_Dhcp_COMMAND );
	DEFENV( CFG_ENV_NFSBOOT_NFS_CMD_NAME, CFG_ENV_NFSBOOT_NFS_COMMAND );
	DEFENV( CFG_ENV_NFSBOOT_COMMON_CMD_NAME, CFG_ENV_NFSBOOT_COMMON_COMMAND );

	setenv( CFG_ENV_TFTP_UPDATE_Dhcp_CMD_NAME, CFG_ENV_TFTP_UPDATE_Dhcp_COMMAND );
	setenv( CFG_ENV_TFTP_UPDATE_CMD_NAME, CFG_ENV_TFTP_UPDATE_COMMAND );
	setenv( CFG_ENV_WRITE_FLASH_CMD_NAME, CFG_ENV_WRITE_FLASH_COMMAND );
	setenv( CFG_ENV_FIX_RCW_ENABLE_NAME, NULL );	/* unset fix_rcw_enable */
	setenv( CFG_ENV_FIX_RCW_CMD_NAME, CFG_ENV_FIX_RCW_COMMAND );

	DEFENV( CFG_ENV_NFS_UPDATE_NOR_Dhcp_CMD_NAME, CFG_ENV_NFS_UPDATE_NOR_Dhcp_COMMAND );
	DEFENV( CFG_ENV_NFS_UPDATE_NOR_CMD_NAME, CFG_ENV_NFS_UPDATE_NOR_COMMAND );
	DEFENV( CFG_ENV_NFS_UPDATE_NOR_IMG_CMD_NAME, CFG_ENV_NFS_UPDATE_NOR_IMG_COMMAND );
	DEFENV( CFG_ENV_NFS_UPDATE_NOR_INITRD_CMD_NAME, CFG_ENV_NFS_UPDATE_NOR_INITRD_COMMAND );

	DEFENV( CFG_ENV_MTEST_PATTERN_NAME, CFG_ENV_MTEST_PATTERN_DFL );
	DEFENV( CFG_ENV_MEMTEST_CMD_NAME, CFG_ENV_MEMTEST_COMMAND );
	return;
}

/* --- */
static void	setenv_by_eeprom( const eepromflags_t *eeprom_flags )
{
	DECLARE_GLOBAL_DATA_PTR;

	int		boot_target;
	char	buf[ CFG_BELLINI_INIT_R_WORKBUF_SIZE ];

	setenv_macaddr( CFG_ENVNAME_ETH0, sizeof(eeprom_flags->mac0_addr), eeprom_flags->mac0_addr );
	memcpy( gd->bd->bi_enetaddr, eeprom_flags->mac0_addr, sizeof( gd->bd->bi_enetaddr ) );
	setenv_macaddr( CFG_ENVNAME_ETH1, sizeof(eeprom_flags->mac1_addr), eeprom_flags->mac1_addr );
	memcpy( gd->bd->bi_enet1addr, eeprom_flags->mac1_addr, sizeof( gd->bd->bi_enet1addr ) );

	boot_target = decide_target( eeprom_flags );
	sprintf( buf, CFG_ENV_BOOTCMDCOMMAND_FMT
				, ( boot_target == BEL_BOOT_ERROR ) ? "" : CFG_ENV_AUTOSETUP_CMD_NAME
				, boot_target );
	setenv( CFG_ENV_BOOTCMD_NAME, buf );
	return;
}

#define	CHARS_IN_OCTET	2
/* --- */
static void	setenv_macaddr( char *varname, int sizeof_field, const uint8_t *mac_addr )
{
	char	buf[ CFG_BELLINI_INIT_R_WORKBUF_SIZE ];
	char	buf_octet[ CHARS_IN_OCTET + 1 + 1 ];
	int		idx;

	buf[0] = '\0';
	for( idx = 0; idx < sizeof_field; ++idx )
	{
		sprintf( buf_octet, ":%02X", mac_addr[idx] );
		strcat( buf, buf_octet );
	}
	setenv( varname, &(buf[1]) );	/* &[1] for cutting off the top of ':' */
	return;
}
#undef	CHARS_IN_OCTET

/* --- */
static int	decide_target( const eepromflags_t *eeprom_flags )
{
	int	boot_target = eeprom_flags->boot_sel;

	if( (eeprom_flags->f_diag_pass) != BEL_DIAGFLAG_PASS )
	{
		boot_target = BEL_BOOT_DIAG;
	}
	else if( boot_target >= NUM_ID_BOOTFLAG )
	{
		boot_target = BEL_BOOT_ERROR;
	}

#if defined( CONFIG_BOOTSELECTKEY )
	{
		int	sel_by_key;

		sel_by_key = boot_target;
		while( ftstc( stdin ) != BV_FALSE )
		{
			switch( fgetc( stdin ) )
			{
			  case CFG_BOOTSELKEY_EMERGENCY:
				sel_by_key = BEL_BOOT_ERROR;
				break;
			  case CFG_BOOTSELKEY_MAIN1:
				sel_by_key = BEL_BOOT_MAIN1;
				break;
			  case CFG_BOOTSELKEY_MAIN2:
				sel_by_key = BEL_BOOT_MAIN2;
				break;
			  case CFG_BOOTSELKEY_NOR:
				sel_by_key = BEL_BOOT_DIAG;
				break;
			  case CFG_BOOTSELKEY_MON:
				sel_by_key = BEL_BOOT_MONITOR;
				break;
			  default:
				;	/* unchanged the selection */
				break;
			}
			if( boot_target != sel_by_key )
			{
				boot_target = sel_by_key;
				break;
			}
		}
	}
#endif	/* CONFIG_BOOTSELECTKEY */
	return boot_target;
}


#define ns2clk(ns) (ns / (1000000000 / CONFIG_8349_CLKIN) + 1)

long int initdram (int board_type)
{
	volatile immap_t *im = (immap_t *)CFG_IMMRBAR;
	u32 msize = 0;

	if ((im->sysconf.immrbar & IMMRBAR_BASE_ADDR) != (u32)im)
		return -1;

	/* DDR SDRAM - Main SODIMM */
	im->sysconf.ddrlaw[0].bar = CFG_DDR_BASE & LAWBAR_BAR;
#if defined(CONFIG_SPD_EEPROM)
	msize = spd_sdram(NULL);
#else
	msize = fixed_sdram();
#endif
	/*
	 * Initialize SDRAM if it is on local bus.
	 */
	sdram_init();
	puts("   DDR RAM: ");
	/* return total bus SDRAM size(bytes)  -- DDR */
	return (msize * 1024 * 1024);
}


#if !defined(CONFIG_SPD_EEPROM)
/*************************************************************************
 *  fixed sdram init -- doesn't use serial presence detect.
 ************************************************************************/
int fixed_sdram(void)
{
	volatile immap_t *im = (immap_t *)CFG_IMMRBAR;

	im->sysconf.ddrlaw[0].ar	= CFG_DDRLAWAR0_AR;
//	im->sysconf.ddrlaw[1].ar	= CFG_DDRLAWAR_AR_UNUSED( )(;

/* 8347PC : Only one CS0 for DDR on Bellini platform */
	im->ddr.cs_config[0]		= CFG_CSCONFIG0;
	im->ddr.csbnds[0].csbnds	= CFG_CSBNDS0;
//	im->ddr.cs_config[1]		= CFG_DDR_CONFIG_UNUSED( );	/* System default */
//	im->ddr.cs_config[2]		= CFG_DDR_CONFIG_UNUSED( );
//	im->ddr.cs_config[3]		= CFG_DDR_CONFIG_UNUSED( );

	im->ddr.timing_cfg_3		= CFG_DDR_TIMING_3;
	im->ddr.timing_cfg_1		= CFG_DDR_TIMING_1;
	im->ddr.timing_cfg_2		= CFG_DDR_TIMING_2;
	im->ddr.timing_cfg_0		= CFG_DDR_TIMING_0;
	im->ddr.sdram_cfg			= CFG_DDR_CONTROL;
	im->ddr.sdram_cfg_2			= CFG_SDRAM_CFG2;
	im->ddr.sdram_mode			= CFG_DDR_MODE;
	im->ddr.sdram_interval		= CFG_DDR_INTERVAL;
	im->ddr.sdram_clk_cntl		= CFG_SDRAM_CLK_CNTL;

	udelay(200);

	im->ddr.sdram_cfg |= SDRAM_CFG_MEM_EN;

	return CFG_DDR_SIZE;
}
#endif/*!CFG_SPD_EEPROM*/


int checkboard (void)
{
	puts("Board: Sony Bellini platform\n");
	return 0;
}

#if defined(CONFIG_PCI)
/*
 * Initialize PCI Devices, report devices found
 */
#ifndef CONFIG_PCI_PNP
static struct pci_config_table pci_mpc83xxads_config_table[] = {
	{PCI_ANY_ID,PCI_ANY_ID,PCI_ANY_ID,PCI_ANY_ID,
	pci_cfgfunc_config_device, {PCI_ENET0_IOADDR,
				    PCI_ENET0_MEMADDR,
				    PCI_COMMON_MEMORY | PCI_COMMAND_MASTER
	} },
	{}
}
#endif


volatile static struct pci_controller hose[] = {
	{
#ifndef CONFIG_PCI_PNP
	config_table:pci_mpc83xxads_config_table,
#endif
	},
	{
#ifndef CONFIG_PCI_PNP
	config_table:pci_mpc83xxads_config_table,
#endif
	}
};
#endif /* CONFIG_PCI */


void
pci_init_board(void)
{
#ifdef CONFIG_PCI
	extern void pci_mpc83xx_init(volatile struct pci_controller *hose);

	pci_mpc83xx_init(hose);
#endif /* CONFIG_PCI */
}


/*
 * Initialize SDRAM memory on the Local Bus.
 */
void
sdram_init(void)
{
	puts("SDRAM on Local Bus is NOT available!\n");
}


#if defined( CONFIG_SETUP_UPM )
/* 
	Following functions are assumed to be called from cpu/mpc83xx/cpu_init.c, 
	therefore functions accessing console, e.g. puts() cannot be used here (because before initialize).
*/
#define	SET_UPM_RAM( lbus_varname, machine_name ) \
	(lbus_varname).M##machine_name##MR_REGNAME	= ( MxMR_OP_WRITE << MxMR_OP_SHIFT ) | ((0) & MxMR_MAD_MASK); \
	(lbus_varname).bank[CFG_UPM_DUMMY_BR_BANK].br	= \
					 ( 0 \
					 	| (CFG_UPM_DUMMY_BA & BR_BA) \
						| (BR_PS_32 << BR_PS_SHIFT) \
						| (( BR_MSEL_UPM##machine_name ) << BR_MSEL_SHIFT) \
						| (BR_VALID_YES << BR_V_SHIFT) \
					); \
	writeUPMtable( &((lbus_varname).mdr), (u32 *)UPM##machine_name##Table, sizeof(UPM##machine_name##Table)/sizeof(UPM##machine_name##Table[0]) ); \
	(lbus_varname).M##machine_name##MR_REGNAME	= M##machine_name##MR; \
	;
#define	MxMR_OP_NORMAL	0
#define	MxMR_OP_WRITE	1
#define	MxMR_OP_READ	2
#define	MxMR_OP_RUN		3
#define	MxMR_OP_SHIFT	28
#define	MxMR_MAD_MASK	0x0000003F
#define	MxMR_MAD_SHIFT	0
#define	MAMR_REGNAME	mamr
#define	MBMR_REGNAME	mbmr
#define	MCMR_REGNAME	mcmr

void	writeUPMtable( volatile u32 *mxmr, const u32 *tbl, int num_param );

void	setupUPM( void )
{
#include	"Bellini_UPMtables.c"
/* 
  #include binds C code output in report editor screen of Freescale's UPMToolPQIII.exe.
	Following variables are assumed to be declared and defined their values.
		unsigned long MAMR;
		unsigned long MBMR;
		unsigned long MCMR;
		static unsigned long UPMATable[];
		static unsigned long UPMBTable[];
		static unsigned long UPMCTable[];
	When there is unused machine table, you have to omit descriptions from <target>_UPMtables.c by hand.
	If you leave unused tables, although the target will work, but it costs waste memory space.
*/
	volatile immap_t	*immap	= (immap_t *)CFG_IMMRBAR;
	u32		bak_lblawbar;
	u32		bak_lblawar;
	u32		bak_ba;
	u32		bak_or;

	bak_lblawbar	= immap->sysconf.lblaw[CFG_UPM_DUMMY_LBLAW_BANK].bar;
	bak_lblawar		= immap->sysconf.lblaw[CFG_UPM_DUMMY_LBLAW_BANK].ar;
	bak_or			= immap->lbus.bank[CFG_UPM_DUMMY_BR_BANK].or;
	bak_ba			= immap->lbus.bank[CFG_UPM_DUMMY_BR_BANK].br;
	immap->sysconf.lblaw[CFG_UPM_DUMMY_LBLAW_BANK].bar	= CFG_UPM_DUMMY_LBLAW_BA;
	immap->sysconf.lblaw[CFG_UPM_DUMMY_LBLAW_BANK].ar	= (1 << LBLAWAR_EN_SHIFT) | (( CFG_UPM_DUMMY_LBLAW_SIZE_LOG2 - 1 ) << LBLAWAR_SIZE_SHIFT);
	immap->lbus.bank[CFG_UPM_DUMMY_BR_BANK].or			= CFG_UPM_DUMMY_BR_SIZE;
	/* lbiu.br is not need set here, they're set in SET_UPM_RAM() macro (defined above in this module) */

#if defined( CONFIG_USE_UPMA )
	SET_UPM_RAM( immap->lbus, A );
#endif	/* CONFIG_USE_UPMA */
#if defined( CONFIG_USE_UPMB )
	SET_UPM_RAM( immap->lbus, B );
#endif	/* CONFIG_USE_UPMB */
#if defined( CONFIG_USE_UPMC )
	SET_UPM_RAM( immap->lbus, C );
#endif	/* CONFIG_USE_UPMC */

	immap->lbus.bank[CFG_UPM_DUMMY_BR_BANK].br			= bak_ba;
	immap->lbus.bank[CFG_UPM_DUMMY_BR_BANK].or			= bak_or;
	immap->sysconf.lblaw[CFG_UPM_DUMMY_LBLAW_BANK].bar	= bak_lblawbar;
	immap->sysconf.lblaw[CFG_UPM_DUMMY_LBLAW_BANK].ar	= bak_lblawar;
	return;
}

void	writeUPMtable( volatile u32 *mxmr, const u32 *tbl, int num_param )
{
	volatile u32	*dummy = (u32 *)CFG_UPM_DUMMY_BA;

	while( num_param-- > 0 )
	{
		*mxmr = *(tbl++);
		*dummy = 0;
	}
	return;
}
#endif	/* CONFIG_SETUP_UPM */
