/*
 * Header file for Bellini specific procedures in Bellini.c
 * 
 * Copyright (C) 2008 Sony Corporation
 * Maintainer: AKA <masateru.akachi@jp.sony.com>
 * 
 * Change log:
 * 20080526:	AKA <masateru.akachi@jp.sony.com>
 *          	*release
 * 20080912:	AKA <masateru.akachi@jp.sony.com>
 *          	*fixed/added defs and macros related to MEMTEST
 */

#if !defined( BELLINI_LOCAL_H )
#define BELLINI_LOCAL_H

/* Recursive includes */
  /* nothing */

#if defined( __cplusplus )
extern "C"
{
#endif  /* __cplusplus */

/* Definitions */
enum ID_BootFlag_tag
{
	BEL_BOOT_ERROR = 0,
	BEL_BOOT_MONITOR,
	BEL_BOOT_DIAG,
	BEL_BOOT_MAIN1,
	BEL_BOOT_MAIN2,
	NUM_ID_BOOTFLAG,
};
enum FreqMode_BootFlag_tag
{
	BEL_DISPFREQ_50HZ	= 0,
	BEL_DISPFREQ_60HZ,
	NUM_FREQMODE_BOOTFLAG,
};

#define	CFG_BELLINI_ADDR_FREQMODE_REG	0xcf0000e0
#define	CFG_BELLINI_DATA_FREQMODE_50HZ	0x00
#define	CFG_BELLINI_DATA_FREQMODE_60HZ	0x03	/* default */

#define	CFG_BELLINI_INIT_R_WORKBUF_SIZE	80


/* Type-definitions (and relevant definitions) */
typedef	unsigned int				I2C_ADDR_t;

#define	SIZ_MAC_ADDR				6
#define	CFG_EEPROM_ID_BASE			0x01A0
typedef	struct cfg_eeprom_id_tag	CFG_EEPROM_ID_t;
struct cfg_eeprom_id_tag
{
	uint8_t		preamble[8];				/**< String: "Sony PCS"		(+ 0x00-0x07) */
	uint8_t		mac_add_1[SIZ_MAC_ADDR];	/**< MAC addr(BigEndian)	(+ 0x08-0x0D) */
	uint8_t		dmy_1[2];					/**< Filler1(0xFF,0xFF)		(+ 0x0E-0x0F) */
	uint8_t		mac_add_2[SIZ_MAC_ADDR];	/**< MAC addr(BE)			(+ 0x10-0x15) */
	uint8_t		dmy_2[2];					/**< Filler2(0xFF,0xFF)		(+ 0x16-0x17) */
	uint32_t	destination;				/**< ProductDest			(+ 0x18-0x1B) */
	uint32_t	model_id;					/**< Model ID				(+ 0x1C-0x1F) */
	uint32_t	serial_number;				/**< SerialNumber			(+ 0x20-0x23) */
	uint8_t		rsv[8];						/**< reserved				(+ 0x24-0x2B) */
} cfg_eeprom_id_t;     /* 0x01A0-0x01CB */

#define	CFG_EEPROM_BOOT_BASE		0x01E0
typedef	struct cfg_eeprom_boot_tag	CFG_EEPROM_BOOT_t;
struct cfg_eeprom_boot_tag
{
	uint8_t	preamble[4];	/**< String: "Boot"	(+ 0x00-0x03) */
	uint8_t	boot_mode;		/**< mode			(+ 0x04)	*/
	uint8_t	frequency_mode;	/**< frequency mode	(+ 0x05)	*/
	uint8_t	rsv[6];			/**< reserved		(+ 0x06-0x0B) */
};	/* 0x01E0-0x01EB */

#define	CFG_EEPROM_DIAG_BASE		0x01F0
typedef struct cfg_eeprom_diag_tag	CFG_EEPROM_DIAG_t;
struct cfg_eeprom_diag_tag
{
	uint8_t	preamble[4];	/**< String: "Diag"	(+ 0x00-0x03) */
	uint8_t	diag_pass;		/**< DiagFlag		(+ 0x04)      */
#define	BEL_DIAGFLAG_PASS	0
	uint8_t	rsv[7];			/**< reserved		(+ 0x05-0x0B) */
};	/* 0x01E0-0x01FB */

typedef	struct eepromflags_tag	eepromflags_t;
struct eepromflags_tag
{	/* All are raw values in EEP-ROM */
	int		boot_sel;					/* Target OS ID. This keeps seed value, not modified by select key or so */
	int		f_diag_pass;				/* Flag:Diagnostic pass: BEL_DIAGFLAG_PASS=OK/Otherwise=NG */
	int		vfreq_mode;					/* VideoFreq mode: BEL_DISPFREQ_50HZ=50Hz/Otherwise=60Hz */
	uint8_t	mac0_addr[SIZ_MAC_ADDR];	/* MAC address for eth0, big-endian order */
	uint8_t	mac1_addr[SIZ_MAC_ADDR];	/* MAC address for eth1, BE order */
};

typedef struct tsec_rx_fifo_tag	TSEC_RX_FIFO_t;
struct	tsec_rx_fifo_tag
{
	uint8_t		rsv[ 0x050 ];
	uint32_t	alarm;
	uint32_t	alarm_shutoff;
	uint32_t	panic;
	uint32_t	panic_shutoff;
};


/* Macros */
#define	CTRL_CODE( sign_char )	((sign_char) - '@')	/* CTRL_CODE('A') means ^A(==0x01), valid in ASCII char-set*/

/* Prototypes */
/* --- */
static int	get_eeprom_flag( eepromflags_t *eeprom_flags );

static void	set_vfreqmode( const eepromflags_t *eeprom_flags );
static void	overwrite_env( const eepromflags_t *eeprom_flags );

static void	setenv_uboot_parameters( void );
static void	setenv_scripts( void );
static void	setenv_by_eeprom( const eepromflags_t *eeprom_flags );

static void	setenv_macaddr( char *varname, int sizeof_field, const uint8_t *mac_addr );
/* varname cannot take "const" qualifier for setenv() limitation */
static int	decide_target( const eepromflags_t *eeprom_flags );

/* Definitions for environment variable setting */
#define	CFG_ENV_LOADADDR_DFL_NAME		"loadaddr_dfl"
#define	CFG_ENV_LOADADDR_DFL_COMMAND( addr )	CFG_ENV_LOADADDR_DFL_COMMAND_( addr )
#define	CFG_ENV_LOADADDR_DFL_COMMAND_( addr )	#addr

#define	CFG_ENV_MONITOR_SADDR_NAME	"mon_start"
#define	CFG_ENV_ENV_SADDR_NAME		"env_start"
#define	CFG_ENV_ENV_SIZE_NAME		"env_size"
#define	CFG_ENV_NORIMAGE_ADDR_NAME	"addr_nor_img"
#define	CFG_ENV_NORIMAGE_ADDR		"fe020000"
#define	CFG_ENV_NORINITRD_ADDR_NAME	"addr_nor_initrd"
#define	CFG_ENV_NORINITRD_ADDR		"fe200000"

#define	CFG_ENV_MTEST_START_NAME	"memtest_start"
#define	CFG_ENV_MTEST_END_NAME		"memtest_end"
#define	CFG_ENV_MTEST_PATTERN_NAME	"memtest_pattern"
#define	CFG_ENV_MTEST_PATTERN_DFL	"0"
#define	DEF_MEMTEST_VECTOR_START	0x00000000
#define	DEF_MEMTEST_VECTOR_RESET	0x00000100

#define	CFG_ENV_AUTOSETUP_CMD_NAME	"autosetup"
#define	CFG_ENV_AUTOSETUP_COMMAND \
	"echo Booting Bellini system. Please wait for a while..." \
	/* CFG_ENV_AUTOSETUP_COMMAND */

#define	CFG_ENV_BOOTCMD_NAME		"bootcmd"
#define	CFG_ENV_BOOTCMDCOMMAND_FMT	"run %s boot%i"

#define	CFG_ENV_BOOT0_CMD_NAME	"boot0"
#define	CFG_ENV_BOOT0_COMMAND \
	"echo ** Malfunction condition has been detected in boot sequence **" \
	/* CFG_ENV_BOOT0_COMMAND */
#define	CFG_ENV_BOOT1_CMD_NAME	"boot1"
#define	CFG_ENV_BOOT1_COMMAND \
	"echo <<Welcome to Bellini monitor>>" \
	/* CFG_ENV_BOOT1_COMMAND */
#define	CFG_ENV_BOOT2_CMD_NAME	"boot2"
#define	CFG_ENV_BOOT2_COMMAND \
	CFG_SHVAR_ROOTDEV_NAME"='/dev/ram0 rw';"CFG_SHVAR_ROOTFS_NAME"=" \
		";"CFG_SHVAR_INITRD_ADDR_NAME"=${"CFG_ENV_NORINITRD_ADDR_NAME"}" \
		";setenv loadaddr ${"CFG_ENV_NORIMAGE_ADDR_NAME"}" \
	";" \
	"run "CFG_ENV_SETFTRANS_NONE_CMD_NAME" "CFG_ENV_BOOTSTRAP_CMD_NAME \
	/* CFG_ENV_BOOT2_COMMAND */
#define	CFG_ENV_BOOT3_CMD_NAME	"boot3"
#define	CFG_ENV_BOOT3_COMMAND \
	CFG_SHVAR_ROOTDEV_NAME"='/dev/hda2 ro';"CFG_SHVAR_ROOTFS_NAME"='rootfstype=squashfs'" \
		";"CFG_SHVAR_INITRD_ADDR_NAME"=" \
		";"CFG_SHVAR_BOOTPARTITION_NAME"=1" \
	";" \
	"run "CFG_ENV_SETFTRANS_ATA_CMD_NAME" "CFG_ENV_BOOTSTRAP_CMD_NAME \
	/* CFG_ENV_BOOT3_COMMAND */
#define	CFG_ENV_BOOT4_CMD_NAME	"boot4"
#define	CFG_ENV_BOOT4_COMMAND \
	CFG_SHVAR_ROOTDEV_NAME"='/dev/hda6 ro';"CFG_SHVAR_ROOTFS_NAME"='rootfstype=squashfs'" \
		";"CFG_SHVAR_INITRD_ADDR_NAME"=" \
		";"CFG_SHVAR_BOOTPARTITION_NAME"=5" \
	";" \
	"run "CFG_ENV_SETFTRANS_ATA_CMD_NAME" "CFG_ENV_BOOTSTRAP_CMD_NAME \
	/* CFG_ENV_BOOT4_COMMAND */

#define	CFG_ENV_FTRANS_CMD_NAME			"bs_ftrans"
#define	CFG_ENV_SETFTRANS_NONE_CMD_NAME	"bs_setftrans_none"
#define	CFG_ENV_SETFTRANS_NONE_COMMAND \
	"setenv "CFG_ENV_FTRANS_CMD_NAME" ' '" \
	/* CFG_ENV_SETFTRANS_NONE_COMMAND */
#define	CFG_ENV_SETFTRANS_ATA_CMD_NAME	"bs_setftrans_ata"
#define	CFG_ENV_SETFTRANS_ATA_COMMAND \
	"setenv "CFG_ENV_FTRANS_CMD_NAME" diskboot ${loadaddr} 0:${"CFG_SHVAR_BOOTPARTITION_NAME"}" \
	/* CFG_ENV_SETFTRANS_ATA_COMMAND */
#define	CFG_ENV_SETFTRANS_TFTP_CMD_NAME	"bs_setftrans_tftp"
#define	CFG_ENV_SETFTRANS_TFTP_COMMAND \
	"setenv "CFG_ENV_FTRANS_CMD_NAME" tftpboot ${loadaddr} ${"CFG_SHVAR_LOADFNAME_TFTP_NAME"}" \
	/* CFG_ENV_SETFTRANS_TFTP_COMMAND */
#define	CFG_ENV_SETFTRANS_NFS_CMD_NAME	"bs_setftrans_nfs"
#define	CFG_ENV_SETFTRANS_NFS_COMMAND \
	"setenv "CFG_ENV_FTRANS_CMD_NAME" nfs ${loadaddr} ${"CFG_SHVAR_LOADFNAME_NFS_NAME"}" \
	/* CFG_ENV_SETFTRANS_NFS_COMMAND */

#define	CFG_ENV_BOOTSTRAP_CMD_NAME		"bs_bootstrap"
#define	CFG_ENV_BOOTSTRAP_COMMAND \
	"setenv bootargs " \
		"root=${"CFG_SHVAR_ROOTDEV_NAME"} " \
		"${"CFG_SHVAR_ROOTFS_NAME"} " \
		"console=${consoledev},${baudrate} " \
		"${"CFG_SHVAR_NFSROOT_NAME"} " \
		"${othbootargs};" \
	"run "CFG_ENV_FTRANS_CMD_NAME \
	"&&" \
		"run "CFG_ENV_BOOT_HOOK_CMD_NAME \
	";" \
	"setenv loadaddr ${"CFG_ENV_LOADADDR_DFL_NAME"};"CFG_SHVAR_INITRD_ADDR_NAME"=;"CFG_SHVAR_NFSROOT_NAME"=" \
	/* CFG_ENV_BOOTSTRAPCOMMAND */
#define	CFG_ENV_BOOT_HOOK_CMD_NAME		"bs_boot_hook"
#define	CFG_ENV_BOOT_HOOK_COMMAND \
	"bootm ${loadaddr} ${"CFG_SHVAR_INITRD_ADDR_NAME"}" \
	/* CFG_ENV_BOOT_HOOK_COMMAND */

#define	CFG_ENV_ERASE_ENV_CMD_NAME	"erase_env"
#define	CFG_ENV_ERASE_ENV_COMMAND \
	"protect off ${"CFG_ENV_ENV_SADDR_NAME"} +${"CFG_ENV_ENV_SIZE_NAME"}" \
	"&&" \
		"erase ${"CFG_ENV_ENV_SADDR_NAME"} +${"CFG_ENV_ENV_SIZE_NAME"}" \
	";" \
	"protect on ${"CFG_ENV_ENV_SADDR_NAME"} +${"CFG_ENV_ENV_SIZE_NAME"}" \
	/* CFG_ENV_ERASE_ENV_COMMAND */


#define	CFG_ENV_SERVERIP_NAME				"serverip"
#define	CFG_ENV_SERVERIP_UBOOT_BINARY_NAME	"serverip_ubootbin"
#define	CFG_ENV_SERVERIP_TFTP_NAME			"serverip_tftp"
#define	CFG_ENV_SERVERIP_NFS_NAME			"serverip_nfs"

#define	CFG_ENV_PATH_ROOT_NAME				"rootpath"
#define	CFG_ENV_IMAGENAME_BOOT_COMPAT_NAME	"bootfile"
#define	CFG_ENV_IMAGENAME_BOOTSTRAP_NAME	"image_boot"
#define	CFG_ENV_IMAGENAME_UBOOT_BINARY_NAME	"image_ubootbin"
#define	CFG_ENV_IMAGENAME_NOR_NAME			"image_nor"
#define	CFG_ENV_IMAGENAME_NOR_INITRD_NAME	"image_nor_initrd"

#define	CFG_SHVAR_ROOTDEV_NAME			"SV_ROOTDEV"
#define	CFG_SHVAR_ROOTFS_NAME			"SV_ROOTFS"
#define	CFG_SHVAR_INITRD_ADDR_NAME		"SV_ADDR_INITRD"
#define	CFG_SHVAR_BOOTPARTITION_NAME	"SV_BOOT_PART"
#define	CFG_SHVAR_LOADFNAME_TFTP_NAME	"SV_LDFN_TFTP"
#define	CFG_SHVAR_LOADFNAME_NFS_NAME	"SV_LDFN_NFS"
#define	CFG_SHVAR_PRIOR_SERVERIP_NAME	"SV_PRIOR_IP"
#define	CFG_SHVAR_NFSROOT_NAME			"SV_NFSROOT"
#define	CFG_SHVAR_FLWRITE_SRC_ADDR_NAME	"SV_FLW_ADDR_SRC"
#define	CFG_SHVAR_FLWRITE_DST_ADDR_NAME	"SV_FLW_ADDR_DST"
#define	CFG_SHVAR_FLWRITE_SIZE_NAME		"SV_FLW_SIZE"
#define	CFG_SHVAR_FLWRITE_F_CHK_P_NAME	"SV_FLW_CHK_PROTECT"	/* flag: defined or not */
#define	CFG_SHVAR_NFS_SERVERPATH_NAME	"SV_NFS_SPATH"
#define	CFG_SHVAR_BOOTIMG_TRANS			"SV_BOOTIMG_TRANS"

#define	CFG_SHVAR_DHCP_RESULT	"SV_RES_DHCP"
#define	CFG_ENV_Dhcp_CMD_NAME	"Dhcp"
#define	CFG_ENV_Dhcp_COMMAND \
	"dhcp" \
	";" \
	CFG_SHVAR_DHCP_RESULT"=$?" \
	";" \
	"if test ${"CFG_SHVAR_DHCP_RESULT"} -eq 0;" \
	"then " \
		"if test x${"CFG_SHVAR_PRIOR_SERVERIP_NAME"} != x;" \
		"then " \
			"setenv "CFG_ENV_SERVERIP_NAME" ${"CFG_SHVAR_PRIOR_SERVERIP_NAME"}" \
		";" \
		"fi" \
	";" \
	"fi" \
	";" \
	CFG_SHVAR_PRIOR_SERVERIP_NAME"=" \
	";" \
	"test ${"CFG_SHVAR_DHCP_RESULT"} -eq 0"	/* set result value to $? */ \
	/* CFG_ENV_Dhcp_COMMAND */

#define	CFG_ENV_NFSBOOT_Dhcp_CMD_NAME	"Nfsboot"
#define	CFG_ENV_NFSBOOT_Dhcp_COMMAND	"run " CFG_ENV_NFSBOOT_TFTP_Dhcp_CMD_NAME
#define CFG_ENV_NFSBOOT_CMD_NAME		"nfsboot"
#define CFG_ENV_NFSBOOT_COMMAND			"run " CFG_ENV_NFSBOOT_TFTP_CMD_NAME
#define	CFG_ENV_NFSBOOT_TFTP_Dhcp_CMD_NAME	"Nfsboot_tftp"
#define	CFG_ENV_NFSBOOT_TFTP_Dhcp_COMMAND \
	"if test x${"CFG_ENV_SERVERIP_TFTP_NAME"} != x;" \
	"then " \
		CFG_SHVAR_PRIOR_SERVERIP_NAME"=${"CFG_ENV_SERVERIP_TFTP_NAME"}" \
	";" \
	"elif test x${"CFG_ENV_SERVERIP_NFS_NAME"} != x;" \
	"then " \
		CFG_SHVAR_PRIOR_SERVERIP_NAME"=${"CFG_ENV_SERVERIP_NFS_NAME"}" \
	";" \
	"else " \
		CFG_SHVAR_PRIOR_SERVERIP_NAME"=" \
	";" \
	"fi" \
	";" \
	"run "CFG_ENV_Dhcp_CMD_NAME" "CFG_ENV_NFSBOOT_TFTP_CMD_NAME \
	/* CFG_ENV_NFSBOOT_TFTP_Dhcp_COMMAND */
#define CFG_ENV_NFSBOOT_TFTP_CMD_NAME	"nfsboot_tftp"
#define CFG_ENV_NFSBOOT_TFTP_COMMAND \
	"if test x${"CFG_ENV_IMAGENAME_BOOTSTRAP_NAME"} != x;" \
	"then " \
		CFG_SHVAR_LOADFNAME_TFTP_NAME"=${"CFG_ENV_IMAGENAME_BOOTSTRAP_NAME"}" \
	";" \
	"else " \
		CFG_SHVAR_LOADFNAME_TFTP_NAME"=${"CFG_ENV_IMAGENAME_BOOT_COMPAT_NAME"}" \
	";" \
	"fi" \
	";" \
	CFG_SHVAR_BOOTIMG_TRANS"="CFG_ENV_SETFTRANS_TFTP_CMD_NAME \
	";" \
	"run "CFG_ENV_NFSBOOT_COMMON_CMD_NAME \
	/* CFG_ENV_NFSBOOT_TFTP_COMMAND */
#define	CFG_ENV_NFSBOOT_NFS_Dhcp_CMD_NAME	"Nfsboot_nfs"
#define	CFG_ENV_NFSBOOT_NFS_Dhcp_COMMAND \
	"if test x${"CFG_ENV_SERVERIP_NFS_NAME"} != x;" \
	"then " \
		CFG_SHVAR_PRIOR_SERVERIP_NAME"=${"CFG_ENV_SERVERIP_NFS_NAME"}" \
	";" \
	"else " \
		CFG_SHVAR_PRIOR_SERVERIP_NAME"=" \
	";" \
	"fi" \
	";" \
	"run "CFG_ENV_Dhcp_CMD_NAME" "CFG_ENV_NFSBOOT_NFS_CMD_NAME \
	/* CFG_ENV_NFSBOOT_NFS_Dhcp_COMMAND */
#define CFG_ENV_NFSBOOT_NFS_CMD_NAME	"nfsboot_nfs"
#define CFG_ENV_NFSBOOT_NFS_COMMAND \
	CFG_SHVAR_NFS_SERVERPATH_NAME"=${serverip}:${"CFG_ENV_PATH_ROOT_NAME"}" \
	";" \
	"if test x${"CFG_ENV_IMAGENAME_BOOTSTRAP_NAME"} != x;" \
	"then " \
		CFG_SHVAR_LOADFNAME_NFS_NAME"=${"CFG_SHVAR_NFS_SERVERPATH_NAME"}/${"CFG_ENV_IMAGENAME_BOOTSTRAP_NAME"}" \
	";" \
	"else " \
		CFG_SHVAR_LOADFNAME_NFS_NAME"=${"CFG_SHVAR_NFS_SERVERPATH_NAME"}/${"CFG_ENV_IMAGENAME_BOOT_COMPAT_NAME"}" \
	";" \
	"fi" \
	";" \
	CFG_SHVAR_BOOTIMG_TRANS"="CFG_ENV_SETFTRANS_NFS_CMD_NAME \
	";" \
	"run "CFG_ENV_NFSBOOT_COMMON_CMD_NAME \
	/* CFG_ENV_NFSBOOT_TFTP_COMMAND */
#define CFG_ENV_NFSBOOT_COMMON_CMD_NAME	"nfsboot_common"
#define CFG_ENV_NFSBOOT_COMMON_COMMAND \
		CFG_SHVAR_ROOTDEV_NAME"='/dev/nfs rw';"CFG_SHVAR_ROOTFS_NAME"=" \
		";"CFG_SHVAR_NFSROOT_NAME"=\"ip=${ipaddr}:${"CFG_ENV_SERVERIP_NAME"}:${gatewayip}:${netmask}:" \
			"${hostname}:${netdev}:off" \
			" nfsroot=${"CFG_ENV_SERVERIP_NAME"}:${"CFG_ENV_PATH_ROOT_NAME"}\"" \
		";" \
	";" \
	"run ${"CFG_SHVAR_BOOTIMG_TRANS"} "CFG_ENV_BOOTSTRAP_CMD_NAME \
	/* CFG_ENV_NFSBOOT_COMMON_COMMAND */

#define	CFG_ENV_TFTP_UPDATE_Dhcp_CMD_NAME	"Rewrite_uboot_tftp"
#define	CFG_ENV_TFTP_UPDATE_Dhcp_COMMAND \
	"if test x${"CFG_ENV_SERVERIP_UBOOT_BINARY_NAME"} != x;" \
	"then " \
		CFG_SHVAR_PRIOR_SERVERIP_NAME"=${"CFG_ENV_SERVERIP_UBOOT_BINARY_NAME"}" \
	";" \
	"elif test x${"CFG_ENV_SERVERIP_TFTP_NAME"} != x;" \
	"then " \
		CFG_SHVAR_PRIOR_SERVERIP_NAME"=${"CFG_ENV_SERVERIP_TFTP_NAME"}" \
	";" \
	"else " \
		CFG_SHVAR_PRIOR_SERVERIP_NAME"=" \
	";" \
	"fi" \
	";" \
	"run "CFG_ENV_Dhcp_CMD_NAME" "CFG_ENV_TFTP_UPDATE_CMD_NAME \
	/* CFG_ENV_TFTP_UPDATE_Dhcp_COMMAND */
#define	CFG_ENV_TFTP_UPDATE_CMD_NAME	"rewrite_uboot_tftp"
#define	CFG_ENV_TFTP_UPDATE_COMMAND \
	"if test x${"CFG_ENV_IMAGENAME_UBOOT_BINARY_NAME"} != x;" \
	"then " \
		CFG_SHVAR_LOADFNAME_TFTP_NAME"=${"CFG_ENV_IMAGENAME_UBOOT_BINARY_NAME"}" \
	";" \
	"fi" \
	";" \
	"run "CFG_ENV_SETFTRANS_TFTP_CMD_NAME" "CFG_ENV_FTRANS_CMD_NAME \
	";" \
	"if test $? -eq 0;" \
	"then " \
		CFG_SHVAR_FLWRITE_SRC_ADDR_NAME"=${loadaddr}" \
			";"CFG_SHVAR_FLWRITE_DST_ADDR_NAME"=${"CFG_ENV_MONITOR_SADDR_NAME"}" \
			";"CFG_SHVAR_FLWRITE_SIZE_NAME"=${filesize}" \
			";"CFG_SHVAR_FLWRITE_F_CHK_P_NAME"=1" \
		";" \
		"run "CFG_ENV_WRITE_FLASH_CMD_NAME \
	";" \
	"fi" \
	/* CFG_ENV_TFTP_UPDATE_COMMAND */

#define	CFG_SHVAR_WRITE_FLASH_RESULT	"SV_RES_WR_FL"
#define	CFG_ENV_WRITE_FLASH_CMD_NAME	"bs_write_flash"
#define	CFG_ENV_WRITE_FLASH_COMMAND \
	"if test x${"CFG_SHVAR_FLWRITE_F_CHK_P_NAME"} != x;" \
	"then " \
		"protect off ${"CFG_SHVAR_FLWRITE_DST_ADDR_NAME"} +${"CFG_SHVAR_FLWRITE_SIZE_NAME"}" \
	";" \
	"fi" \
	";" \
	"echo Erasing ${"CFG_SHVAR_FLWRITE_SIZE_NAME"} bytes block from ${"CFG_SHVAR_FLWRITE_DST_ADDR_NAME"}... " \
	";" \
	"erase ${"CFG_SHVAR_FLWRITE_DST_ADDR_NAME"} +${"CFG_SHVAR_FLWRITE_SIZE_NAME"}" \
	";" \
	CFG_SHVAR_WRITE_FLASH_RESULT"=$?" \
	";" \
	"if test ${"CFG_SHVAR_WRITE_FLASH_RESULT"} -eq 0;" \
	"then " \
		"echo Copying ${"CFG_SHVAR_FLWRITE_SIZE_NAME"} bytes from Src:${"CFG_SHVAR_FLWRITE_SRC_ADDR_NAME"} " \
				"to Dest:${"CFG_SHVAR_FLWRITE_DST_ADDR_NAME"}"  \
		";" \
		"cp.b ${"CFG_SHVAR_FLWRITE_SRC_ADDR_NAME"} ${"CFG_SHVAR_FLWRITE_DST_ADDR_NAME"}" \
				" ${"CFG_SHVAR_FLWRITE_SIZE_NAME"}" \
		";" \
		CFG_SHVAR_WRITE_FLASH_RESULT"=$?" \
	";" \
	"fi" \
	";" \
	"if test x${"CFG_SHVAR_FLWRITE_F_CHK_P_NAME"} != x;" \
	"then " \
		"protect on ${"CFG_SHVAR_FLWRITE_DST_ADDR_NAME"} +${"CFG_SHVAR_FLWRITE_SIZE_NAME"}" \
		";" \
		CFG_SHVAR_FLWRITE_F_CHK_P_NAME"=" \
	";" \
	"fi" \
	";" \
	"test ${"CFG_SHVAR_WRITE_FLASH_RESULT"} -eq 0"	/* set result value to $? */ \
	/* CFG_ENV_WRITE_FLASH_COMMAND */

#define	CFG_ENV_RCW_ADDR_NAME	"rcw_addr"
#define	CFG_RCW_ADDD			0xfe000000
#define	CFG_ENV_RCW_SIZE_NAME	"rcw_size"
#define	CFG_RCW_SIZE			0x40
#define	CFG_ENV_FIX_RCW_ENABLE_NAME	"fix_rcw_enable"
#define	CFG_ENV_FIX_RCW_CMD_NAME	"fix_rcw"
#define	CFG_ENV_FIX_RCW_COMMAND \
	"if test x${"CFG_ENV_FIX_RCW_ENABLE_NAME"} = x || itest.s x${"CFG_ENV_FIX_RCW_ENABLE_NAME"} -ne xy;" \
	"then " \
		"echo 'If you really want to do, run the following command before it.'" \
		";" \
		"echo '  setenv "CFG_ENV_FIX_RCW_ENABLE_NAME" y'" \
	";" \
	"else " \
		CFG_SHVAR_FLWRITE_SRC_ADDR_NAME"=${"CFG_ENV_MONITOR_SADDR_NAME"}" \
			";"CFG_SHVAR_FLWRITE_DST_ADDR_NAME"=${"CFG_ENV_RCW_ADDR_NAME"}" \
			";"CFG_SHVAR_FLWRITE_SIZE_NAME"=${"CFG_ENV_RCW_SIZE_NAME"}" \
		";" \
		"run "CFG_ENV_WRITE_FLASH_CMD_NAME \
	";" \
	"fi" \
	/* CFG_ENV_FIX_RCW_COMMAND */

#define	CFG_ENV_NFS_UPDATE_NOR_Dhcp_CMD_NAME	"Update_nor_nfs"
#define	CFG_ENV_NFS_UPDATE_NOR_Dhcp_COMMAND \
	"if test x${"CFG_ENV_SERVERIP_NFS_NAME"} != x;" \
	"then " \
		CFG_SHVAR_PRIOR_SERVERIP_NAME"=${"CFG_ENV_SERVERIP_NFS_NAME"}" \
	";" \
	"elif test x${"CFG_ENV_SERVERIP_TFTP_NAME"} != x;" \
	"then " \
		CFG_SHVAR_PRIOR_SERVERIP_NAME"=${"CFG_ENV_SERVERIP_TFTP_NAME"}" \
	";" \
	"else " \
		CFG_SHVAR_PRIOR_SERVERIP_NAME"=" \
	";" \
	"fi" \
	";" \
	"run "CFG_ENV_Dhcp_CMD_NAME" "CFG_ENV_NFS_UPDATE_NOR_CMD_NAME \
	/* CFG_ENV_NFS_UPDATE_NOR_Dhcp_COMMAND */
#define	CFG_ENV_NFS_UPDATE_NOR_CMD_NAME	"update_nor_nfs"
#define	CFG_ENV_NFS_UPDATE_NOR_COMMAND \
	CFG_SHVAR_NFS_SERVERPATH_NAME"=${serverip}:${"CFG_ENV_PATH_ROOT_NAME"}" \
	";" \
	"run "CFG_ENV_NFS_UPDATE_NOR_IMG_CMD_NAME" "CFG_ENV_NFS_UPDATE_NOR_INITRD_CMD_NAME \
	"&&" \
		"echo Updated." \
	/* CFG_ENV_NFS_UPDATE_NOR_COMMAND */

#define	CFG_ENV_NFS_UPDATE_NOR_IMG_CMD_NAME		"update_nor_img"
#define	CFG_ENV_NFS_UPDATE_NOR_IMG_COMMAND \
	CFG_SHVAR_LOADFNAME_NFS_NAME"=${"CFG_SHVAR_NFS_SERVERPATH_NAME"}/${"CFG_ENV_IMAGENAME_NOR_NAME"}" \
	";" \
	"run "CFG_ENV_SETFTRANS_NFS_CMD_NAME" "CFG_ENV_FTRANS_CMD_NAME \
	";" \
	"if test $? -eq 0;" \
	"then " \
		CFG_SHVAR_FLWRITE_SRC_ADDR_NAME"=${loadaddr}" \
			";"CFG_SHVAR_FLWRITE_DST_ADDR_NAME"=${"CFG_ENV_NORIMAGE_ADDR_NAME"}" \
			";"CFG_SHVAR_FLWRITE_SIZE_NAME"=${filesize}" \
		";" \
		"run "CFG_ENV_WRITE_FLASH_CMD_NAME \
	";" \
	"fi" \
	/* CFG_ENV_NFS_UPDATE_NOR_IMG_COMMAND */

#define	CFG_ENV_NFS_UPDATE_NOR_INITRD_CMD_NAME	"update_nor_initrd"
#define	CFG_ENV_NFS_UPDATE_NOR_INITRD_COMMAND \
	CFG_SHVAR_LOADFNAME_NFS_NAME"=${"CFG_SHVAR_NFS_SERVERPATH_NAME"}/${"CFG_ENV_IMAGENAME_NOR_INITRD_NAME"}" \
	";" \
	"run "CFG_ENV_SETFTRANS_NFS_CMD_NAME" "CFG_ENV_FTRANS_CMD_NAME \
	";" \
	"if test $? -eq 0;" \
	"then " \
		CFG_SHVAR_FLWRITE_SRC_ADDR_NAME"=${loadaddr}" \
			";"CFG_SHVAR_FLWRITE_DST_ADDR_NAME"=${"CFG_ENV_NORINITRD_ADDR_NAME"}" \
			";"CFG_SHVAR_FLWRITE_SIZE_NAME"=${filesize}" \
		";" \
		"run "CFG_ENV_WRITE_FLASH_CMD_NAME \
	";" \
	"fi" \
	/* CFG_ENV_NFS_UPDATE_NOR_INITRD_COMMAND */

#define	CFG_ENV_MEMTEST_CMD_NAME	"memtest"
#define	CFG_ENV_MEMTEST_COMMAND \
	"echo Testing memory from ${"CFG_ENV_MTEST_START_NAME"} to ${"CFG_ENV_MTEST_END_NAME"}." \
	";" \
	"echo To quit, type Ctrl-C (only sampled at every top of the test loop)." \
	";" \
	"mtest ${"CFG_ENV_MTEST_START_NAME"} ${"CFG_ENV_MTEST_END_NAME"} ${"CFG_ENV_MTEST_PATTERN_NAME"}" \
	/* CFG_ENV_MEMTEST_COMMAND */

#if defined( __cplusplus )
}
#endif  /* __cplusplus */

#endif  /* ^- !BELLINI_LOCAL_H */
/* --- End of bellini_local.h --- */
