/*
 * Copyright 2004 Freescale Semiconductor, Inc.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 * Change log:
 *
 * 20050101: Eran Liberty (liberty@freescale.com)
 *           Initial file creating (porting from 85XX & 8260)
 * 20071015: AKA <masateru.akachi@jp.sony.com>
 *           *isolated #if-clause enveloped setting functions for LAW and LBIU
 * 				(Fundamentally, these might be independent features.)
 *           *changed hard-coded parameters for SICRH/L to use configurations in <Target>.h
 * 20080428: AKA <masateru.akachi@jp.sony.com>
 *           *accepted "system performance tweaking" patch code from u-boot 1.3.2 tree.
 */

#include <common.h>
#include <mpc83xx.h>
#include <ioports.h>

#if defined( CONFIG_SETUP_UPM )
	extern void	setupUPM( void );
#endif	/* CONFIG_SETUP_UPM */


/*
 * Breathe some life into the CPU...
 *
 * Set up the memory map,
 * initialize a bunch of registers,
 * initialize the UPM's
 */
void cpu_init_f (volatile immap_t * im)
{
	DECLARE_GLOBAL_DATA_PTR;

	/* Pointer is writable since we allocated a register for it */
	gd = (gd_t *) (CFG_INIT_RAM_ADDR + CFG_GBL_DATA_OFFSET);

	/* Clear initial global data */
	memset ((void *) gd, 0, sizeof (gd_t));


	/* system performance tweaking */
#ifdef CFG_ACR_PIPE_DEP
	/* Arbiter pipeline depth */
	im->arbiter.acr = (im->arbiter.acr & ~ACR_PIPE_DEP) |
			  (CFG_ACR_PIPE_DEP << ACR_PIPE_DEP_SHIFT);
#endif

#ifdef CFG_ACR_RPTCNT
	/* Arbiter repeat count */
	im->arbiter.acr = (im->arbiter.acr & ~(ACR_RPTCNT)) |
			  (CFG_ACR_RPTCNT << ACR_RPTCNT_SHIFT);
#endif

#ifdef CFG_SPCR_TSECEP
	/* all eTSEC's Emergency priority */
	im->sysconf.spcr = (im->sysconf.spcr & ~SPCR_TSECEP) |
			   (CFG_SPCR_TSECEP << SPCR_TSECEP_SHIFT);
#endif

#ifdef CFG_SPCR_TSEC1BDP
	/* TSEC1 Buffer Descriptor Priority */
	im->sysconf.spcr = (im->sysconf.spcr & ~SPCR_TSEC1BDP) |
			   (CFG_SPCR_TSEC1BDP << SPCR_TSEC1BDP_SHIFT);
#endif

#ifdef CFG_SPCR_TSEC1EP
	/* TSEC1 Emergency priority */
	im->sysconf.spcr = (im->sysconf.spcr & ~SPCR_TSEC1EP) |
			   (CFG_SPCR_TSEC1EP << SPCR_TSEC1EP_SHIFT);
#endif

#ifdef CFG_SPCR_TSEC2BDP
	/* TSEC2 Buffer Descriptor Priority */
	im->sysconf.spcr = (im->sysconf.spcr & ~SPCR_TSEC2BDP) |
			   (CFG_SPCR_TSEC2BDP << SPCR_TSEC2BDP_SHIFT);
#endif

#ifdef CFG_SPCR_TSEC2EP
	/* TSEC2 Emergency priority */
	im->sysconf.spcr = (im->sysconf.spcr & ~SPCR_TSEC2EP) |
			   (CFG_SPCR_TSEC2EP << SPCR_TSEC2EP_SHIFT);
#endif

#ifdef CFG_SCCR_ENCCM
	/* Encryption clock mode */
	im->clk.sccr = (im->clk.sccr & ~SCCR_ENCCM) |
			   (CFG_SCCR_ENCCM << SCCR_PCICM_SHIFT);
#endif

#ifdef CFG_SCCR_PCICM
	/* PCI & DMA clock mode */
	im->clk.sccr = (im->clk.sccr & ~SCCR_PCICM) |
			   (CFG_SCCR_PCICM << SCCR_PCICM_SHIFT);
#endif

#ifdef CFG_SCCR_TSECCM
	/* all TSEC's clock mode */
	im->clk.sccr = (im->clk.sccr & ~SCCR_TSECCM) |
			   (CFG_SCCR_TSECCM << SCCR_TSECCM_SHIFT);
#endif

#ifdef CFG_SCCR_TSEC1CM
	/* TSEC1 clock mode */
	im->clk.sccr = (im->clk.sccr & ~SCCR_TSEC1CM) |
			   (CFG_SCCR_TSEC1CM << SCCR_TSEC1CM_SHIFT);
#endif

#ifdef CFG_SCCR_TSEC2CM
	/* TSEC2 clock mode */
	im->clk.sccr = (im->clk.sccr & ~SCCR_TSEC2CM) |
			   (CFG_SCCR_TSEC2CM << SCCR_TSEC2CM_SHIFT);
#endif

#ifdef CFG_SCCR_TSEC1ON
	/* TSEC1 clock switch */
	im->clk.sccr = (im->clk.sccr & ~SCCR_TSEC1ON) |
			   (CFG_SCCR_TSEC1ON << SCCR_TSEC1ON_SHIFT);
#endif

#ifdef CFG_SCCR_TSEC2ON
	/* TSEC2 clock switch */
	im->clk.sccr = (im->clk.sccr & ~SCCR_TSEC2ON) |
			   (CFG_SCCR_TSEC2ON << SCCR_TSEC2ON_SHIFT);
#endif

#ifdef CFG_SCCR_USBMPHCM
	/* USB MPH clock mode */
	im->clk.sccr = (im->clk.sccr & ~SCCR_USBMPHCM) |
			   (CFG_SCCR_USBMPHCM << SCCR_USBMPHCM_SHIFT);
#endif

#ifdef CFG_SCCR_USBDRCM
	/* USB DR clock mode */
	im->clk.sccr = (im->clk.sccr & ~SCCR_USBDRCM) |
			   (CFG_SCCR_USBDRCM << SCCR_USBDRCM_SHIFT);
#endif

#ifdef CFG_SCCR_SATACM
	/* SATA controller clock mode */
	im->clk.sccr = (im->clk.sccr & ~SCCR_SATACM) |
			   (CFG_SCCR_SATACM << SCCR_SATACM_SHIFT);
#endif

	/* RSR - Reset Status Register - clear all status (4.6.1.3) */
	gd->reset_status = im->reset.rsr;
	im->reset.rsr = ~(RSR_RES);

	/*
	 * RMR - Reset Mode Register
	 * contains checkstop reset enable (4.6.1.4)
	 */
	im->reset.rmr = (RMR_CSRE & (1<<RMR_CSRE_SHIFT));

	/* LCRR - Clock Ratio Register (10.3.1.16) */
	im->lbus.lcrr = CFG_LCRR;

	/* Enable Time Base & Decrimenter ( so we will have udelay() )*/
	im->sysconf.spcr |= SPCR_TBEN;

	/* System General Purpose Register */
	im->sysconf.sicrh = CFG_SICRH;	/* set as configured value */
	im->sysconf.sicrl = CFG_SICRL;

	/*
	 * Memory Controller:
	 */

	/* Map banks 0 and 1 to the FLASH banks 0 and 1 at preliminary
	 * addresses - these have to be modified later when FLASH size
	 * has been determined
	 */

#define	SET_LBLAW_REGS(banknum) \
			im->sysconf.lblaw[banknum].bar = CFG_LBLAWBAR##banknum##_PRELIM; \
			im->sysconf.lblaw[banknum].ar = CFG_LBLAWAR##banknum##_PRELIM;
#define	SET_LBUS_BANK_REGS(banknum) \
			im->lbus.bank[banknum].br = CFG_BR##banknum##_PRELIM; \
			im->lbus.bank[banknum].or = CFG_OR##banknum##_PRELIM;

#if defined(CFG_BR0_PRELIM)  \
	&& defined(CFG_OR0_PRELIM) \
	&& defined(CFG_LBLAWBAR0_PRELIM) \
	&& defined(CFG_LBLAWAR0_PRELIM)
		SET_LBLAW_REGS(0);
		SET_LBUS_BANK_REGS(0);
#else
#error 	CFG_BR0_PRELIM, CFG_OR0_PRELIM, CFG_LBLAWBAR0_PRELIM & CFG_LBLAWAR0_PRELIM must be defined
#endif

	/* If BAR is defined, OR setting must be also defined, don't you? */
#if defined(CFG_LBLAWBAR1_PRELIM)
	SET_LBLAW_REGS(1);
#endif
#if defined(CFG_LBLAWBAR2_PRELIM)
	SET_LBLAW_REGS(2);
#endif
#if defined(CFG_LBLAWBAR3_PRELIM)
	SET_LBLAW_REGS(3);
#endif

#if defined( CONFIG_SETUP_UPM )
	setupUPM( );
#endif	/* CONFIG_SETUP_UPM */

#if defined(CFG_BR1_PRELIM)
	SET_LBUS_BANK_REGS(1);
#endif
#if defined(CFG_BR2_PRELIM)
	SET_LBUS_BANK_REGS(2);
#endif
#if defined(CFG_BR3_PRELIM)
	SET_LBUS_BANK_REGS(3);
#endif
#if defined(CFG_BR4_PRELIM)
	SET_LBUS_BANK_REGS(4);
#endif
#if defined(CFG_BR5_PRELIM)
	SET_LBUS_BANK_REGS(5);
#endif
#if defined(CFG_BR6_PRELIM)
	SET_LBUS_BANK_REGS(6);
#endif
#if defined(CFG_BR7_PRELIM)
	SET_LBUS_BANK_REGS(7);
#endif
}


/*
 * Initialize higher level parts of CPU like time base and timers.
 */

int cpu_init_r (void)
{
	return 0;
}
