/*
 * Copyright 2004 Freescale Semiconductor.
 * Copyright (C) Sony corporation. 2007.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Sony Bellini platform configuration file
 * Please refer to doc/README.mpc83xxads for more info.
 * 
 * 20071007:	AKA <masateru.akachi@jp.sony.com>
 *          	Initial file creating (porting from include/configs/MPC8349ADS.h)
 * 20080526:	AKA <masateru.akachi@jp.sony.com>
 *          	*adjusted for Mass Production branch
 * 20080912:	AKA <masateru.akachi@jp.sony.com>
 *          	*fixed/added MEMTEST parameter definitions 
 */

#ifndef __CONFIG_H
#define __CONFIG_H

#include	"Bellini_mkconst.h"

/*	#undef DEBUG	*/	/* leave it to option setting in make */

#define CONFIG_MII

/*
 * High Level Configuration Options
 */
#define CONFIG_E300		1	/* E300 Family */
#define CONFIG_MPC83XX		1	/* MPC83XX family */
#define CONFIG_MPC8349		1	/* MPC8349 specific */
#undef	CONFIG_MPC8349ADS	/*	1	[Unused]MPC8349ADS board specific */
#define	CONFIG_BELLINI_EVT	1
#define	CONFIG_BELLINI_DVT	2
#define	CONFIG_BELLINI_PVT	4
#define	CONFIG_BELLINI_MP	0x10
#define	CONFIG_BELLINI		CONFIG_BELLINI_MP	/* Sony Bellini platform specific */
#if CONFIG_BELLINI == CONFIG_BELLINI_EVT
	#define	CFGFLAG_CPU_REV	PVR_8349_REV11
#elif CONFIG_BELLINI <= CONFIG_BELLINI_MP
	#define	CFGFLAG_CPU_REV	PVR_8349_REV31
#else
	#define	CFGFLAG_CPU_REV	0	/* invalid value */
#endif	/* depending on CONFIG_BELLINI */

#if CFGFLAG_CPU_REV <= PVR_8349_REV31
	#define	CONFIG_PATCH_ERRATA_TSEC_6_1
#else
	/* CFGFLAG_CPU_REV indecates fixed errata */
	#undef	CONFIG_PATCH_ERRATA_TSEC_6_1
#endif	/* CMP: CFGFLAG_CPU_REV */

/*#define	CFG_BELLINI_TO_BE_FIXED	1 */
#define	BELLINI_LINUX_PCI_PROBLEM_PATCH
#define	CFG_OCCR_DISABLE_MASK \
			( OCCR_PCICOE0 | OCCR_PCICOE1 |	OCCR_PCICOE2 | OCCR_PCICOE3 \
				| OCCR_PCICOE4 | OCCR_PCICOE5 | OCCR_PCICOE6 | OCCR_PCICOE7 )

/* FIXME: Real PCI support will come in a follow-up update. */
#undef CONFIG_PCI	/* [Unused] */

#if defined( BELLINI_LINUX_PCI_PROBLEM_PATCH )
#undef	CONFIG_PCI
#define CONFIG_PCI

#define CFG_PCI1_MEM_BASE	0x80000000
#define CFG_PCI1_MEM_PHYS	CFG_PCI1_MEM_BASE
#define CFG_PCI1_MEM_SIZE	0x20000000	/* 512M */
#define CFG_PCI1_IO_BASE	0x00000000
#define CFG_PCI1_IO_PHYS	0xe2000000
#define CFG_PCI1_IO_SIZE	0x1000000	/* 16M */

#define CFG_PCI2_MEM_BASE	0xA0000000
#define CFG_PCI2_MEM_PHYS	CFG_PCI2_MEM_BASE
#define CFG_PCI2_MEM_SIZE	0x20000000	/* 512M */
#define CFG_PCI2_IO_BASE	0x00000000
#define CFG_PCI2_IO_PHYS	0xe3000000
#define CFG_PCI2_IO_SIZE	0x1000000	/* 16M */
#if defined(CONFIG_PCI)

#define PCI_ALL_PCI1
#if defined(PCI_64BIT)
#undef PCI_ALL_PCI1
#undef PCI_TWO_PCI1
#undef PCI_ONE_PCI1
#endif

#define CONFIG_NET_MULTI
#define CONFIG_PCI_PNP	               	/* do pci plug-and-play */

#undef CONFIG_EEPRO100
#undef CONFIG_TULIP

#if !defined(CONFIG_PCI_PNP)
	#define PCI_ENET0_IOADDR	0xFIXME
	#define PCI_ENET0_MEMADDR	0xFIXME
	#define PCI_IDSEL_NUMBER	0x0c 	/* slot0->3(IDSEL)=12->15 */
#endif

#undef CONFIG_PCI_SCAN_SHOW		/* show pci devices on startup */
#define CFG_PCI_SUBSYS_VENDORID 0x1057  /* Motorola */

#endif	/* CONFIG_PCI */
#endif	/* BELLINI_LINUX_PCI_PROBLEM_PATCH */


#define CONFIG_TSEC_ENET 		/* tsec ethernet support */
#define CONFIG_ENV_OVERWRITE

#undef	CONFIG_SPD_EEPROM		/* [Unused]Use SPD EEPROM for DDR setup*/

#undef CONFIG_DDR_ECC			/* only for ECC DDR module */

#undef	PCI_66M		/* CLK_IN <= 33MHz */
#ifdef PCI_66M
#define CONFIG_83XX_CLKIN	66000000	/* in Hz */
#else
#define CONFIG_83XX_CLKIN	33000000	/* in Hz */
#endif

#ifndef CONFIG_SYS_CLK_FREQ
#ifdef PCI_66M
#define CONFIG_SYS_CLK_FREQ	66000000
#else
#define CONFIG_SYS_CLK_FREQ	33000000
#endif
#endif

#define	CONFIG_BOARD_EARLY_INIT_F	/* call board_pre_init */
#define	CONFIG_MISC_INIT_R	/* use hook from board initialize function on RAM */

#define CFG_IMMRBAR		0xE0000000

#undef CFG_DRAM_TEST                   /* memory test, takes time */
#define CFG_MEMTEST_START			0x00003000		/* memtest region */
#define CFG_MEMTEST_END				0x1FF00000
#define CFG_MEMTEST_VECTOR_GUARD_SIZE	0x70	/* guard area size after exception vector area */
#define CFG_MEMTEST_STACK_GUARD_SIZE	(1024 * sizeof(ulong))	/* guard before stack top when called setenv_uboot_parameters() */

/*
 * System Configuration
 */
  /* See also TSEC section for handling errata TSEC6.1 */

  /* Note that in SICRL/H, almost set bits mean "Disabled" or "Use alternate". Both default values are 0 */
#define	CFG_SICRL	( SICRL_SPI )
#define	CFG_SICRH	( SICRH_PCI )

/*
 * DDR Setup
 */

#define CFG_DDR_BASE	0x00000000	/* DDR is system memory*/
#define CFG_SDRAM_BASE CFG_DDR_BASE
#define CFG_DDR_SDRAM_BASE CFG_DDR_BASE
#undef  CONFIG_DDR_2T_TIMING		/* [unused] */

#if defined(CONFIG_SPD_EEPROM)
	/*
	 * Determine DDR configuration from I2C interface.
	 */
	#define SPD_EEPROM_ADDRESS	0x51		/* DDR DIMM */
#else
	/*
	 * Manually set up DDR parameters
	 */
#if defined( CONFIG_BELLINI )
#if CONFIG_BELLINI == CONFIG_BELLINI_MP
	#define	CFG_DDR_SIZE_LOG2	29	/* 2^29=512MB */
	#define	CFG_DDR_SIZE		(1 << ( CFG_DDR_SIZE_LOG2 - SIZE_1MB_LOG2 ))	/* SIZE requires in terms of 1MB */

	#define	CSBNDS0_SA		(CFG_DDR_BASE >> ( SIZE_1MB_LOG2 + 4 ))			/* 8MSBs of StartAdr:0x00000000 (in terms of 16MB(=2^4*1MB)) */
	#define	CSBNDS0_EA		((CFG_DDR_SIZE >> 4) - 1)						/* 8MSBs of EndAdr:0x1fffffff */
	#define	CFG_CSBNDS0	MKCFG_CSBNDS(0)

		/*
		 * configurable parameters (# means default value, "Rsv" means "Reserved" value)
		 * Following parameters are adjusted to Elpida EDE1116ACSE-6E @133MHz DDR2 External Bus clock. (Device is capable upto @333MHz)
		 * See Freescale MPC8349E RefManual and Elpida DRAM DataSheet for more details.
		 */
	#define	CFG_DDRLAW0_SIZE_LOG2	CFG_DDR_SIZE_LOG2
	#define	CFG_DDRLAWAR0_AR		MKCFG_DDRLAWAR_DFL(0)

	#define	CFG_CSCONFIG0_EN		1	/* ChipSelect[n] enable-0#:Disable/1:Enable */
	#define	CFG_CSCONFIG0_AP		0	/* Auto-precharge enable-0#:depends BSTOPRE/1:Always issue */
	#define	CFG_CSCONFIG0_ODT_RD	0	/* ODT for read-0#:Never/1:assert only on CS/2:only OtherCS/3:only OtherDIMM/4:All read/5-7:Rsv */
	#define	CFG_CSCONFIG0_ODT_WR	1	/* ODT for write-0#:Never/1:only CS/2:only OtherCS/3:only OtherDIMM/4:All write/5-7:Rsv */
	#define	CFG_CSCONFIG0_BA_BITS	1	/* MBA[] bits-0#:2bits/1:3bits/2-3:Rsv */
	#define	CFG_CSCONFIG0_ROW_BITS	1	/* Row Addr bits-0#:12bit/1:13bit/2:14bit/3:15bit/4-7:Rsv */
	#define	CFG_CSCONFIG0_COL_BITS	2	/* Col Addr bits-0#:8bit/1:9bit/2:10bit/3:11bit/4-7:Rsv */
	#define	CFG_CSCONFIG0		MKCFG_CSCONFIG(0)

	#define	CFG_TIMING_CFG0_RWT			2	/* ReadToWrite turnaround-0#:0clk/1:1clk/.../3:3clk */
	#define	CFG_TIMING_CFG0_WRT			0	/* WriteToRead turnaround-0#:0clk/1:1clk/.../3:3clk */
	#define	CFG_TIMING_CFG0_RRT			0	/* ReadToRead turnaround-0#:0clk/1:1clk/.../3:3clk */
	#define	CFG_TIMING_CFG0_WWT			0	/* WriteToWrite turnaround-0#:0clk/1:1clk/.../3:3clk */
	#define	CFG_TIMING_CFG0_ACT_PD_EXIT	2	/* ActivePowerdownExit-0:Rsv/1#:1clk/2:2clk/.../7:7clk */
	#define	CFG_TIMING_CFG0_PRE_PD_EXIT	2	/* PrechargePowerdownExit-0:Rsv/1#:1clk/2:2clk/.../7:7clk */
	#define	CFG_TIMING_CFG0_ODT_PD_EXIT	8	/* ODT_PowerdownExit-0:Rsv/1#:1clk/2:2clk/.../15:15clk */
	#define	CFG_TIMING_CFG0_MRS_CYC		2	/* ModeRegisterSetCycleTime-0:Rsv/1:1clk/2:2clk/.../5#:5clk/.../15:15clk */
	#define	CFG_DDR_TIMING_0	MKCFG_DDR_TIMING_0( )

	#define	CFG_TIMING_CFG1_PRETOACT	2	/* PrechargeToActivateInterval(tRP)-0#:Rsv/1:1clk/2:2clk/.../7:7clk */
	#define	CFG_TIMING_CFG1_ACTTOPRE	6	/* ActivateToPrechargeInterval(tRAS)-0#:16clk/.../3:19clk/4:4clk/5:5clk/.../15:15clk */
	#define	CFG_TIMING_CFG1_ACTTORW		2	/* ActivateToRd/WrInterval(tRCD)-0#:Rsv/1:1clk/.../7:7clk */
	#define	CFG_TIMING_CFG1_CASLAT		5	/* CAS lateny-0#:Rsv/1:1clk/2:1.5clk/3:2clk/.../13:7clk/14:7.5clk/15:8clk */
	#define	CFG_TIMING_CFG1_REFREC		9	/* RefreshRecoveryTime(tRFC)-0#:8clk/1:9clk/.../14:22clk/15:23clk (see also TimingCfg3[EXT_REFREC] */
	#define	CFG_TIMING_CFG1_WRREC		2	/* LastDataToPrechgMinInterval(tWR)-0#:Rsv/1:1clk/.../7:7clk */
	#define	CFG_TIMING_CFG1_ACTTOACT	2	/* ActToActInterval(tRRD)-0#:Rsv/1:1clk/.../7:7clk */
	#define	CFG_TIMING_CFG1_WRTORD		2	/* LastWriteDataPairToRdCmdIssueInterval(tWTR)-0#:Rsv/1:1clk/.../7:7clk */
	#define	CFG_DDR_TIMING_1	MKCFG_DDR_TIMING_1( )

	#define	CFG_TIMING_CFG2_ADD_LAT			1	/* AdditiveLatency-0#:0clk/1:1clk/2:2clk/.../5:5clk/6-7:Rsv */
	#define CFG_TIMING_CFG2_CPO				0	/* CasToPreambleOverride-0#:RdLat+1/1:Rsv/2:RdLat/3:RdLat+1Q/4:RdLat+2Q/.../21:RdLat+19Q/22-31:Rsv (Q=1/4) */
	#define	CFG_TIMING_CFG2_WR_LAT			2	/* WriteLatency-0#:Rsv/1:1clk/2:2clk/.../7:7clk */
	#define	CFG_TIMING_CFG2_RD_TO_PRE		1	/* ReadToPrecharge-0#:Rsv/1:1cyc/2:2cyc/.../4:4cyc/5-7:Rsv */
	#define CFG_TIMING_CFG2_WR_DATA_DELAY	1	/* WriteCmdToWDataTimingAdjust-0#:0clkDelay/1:1Q/2:2Q/.../6:6Q/7:Rsv (Q=1/4[clk]) */
	#define	CFG_TIMING_CFG2_CKE_PLS			3	/* MinCKEpulseWidth-0#:Rsv/1:1cyc/2:2cyc/.../4:4cyc/5-7:Rsv */
	#define	CFG_TIMING_CFG2_4_ACT			7	/* WindowForFourActivates-0#:Rsv/1:1cyc/2:2cyc/.../20:20cyc/21-63:Rsv */
	#define	CFG_DDR_TIMING_2	MKCFG_DDR_TIMING_2( )

	#define	CFG_TIMING_CFG3_EXT_REFREC	0	/* ExtendedRefreshRecoveryTime-0#:0clk/1:16clk/2:32clk/.../7:112clk (Upper digit of TimingCfg1[REFREC]) */
	#define	CFG_DDR_TIMING_3	MKCFG_DDR_TIMING_3( )

	#define	CFG_SDRAM_CFG_SREN			1	/* SelfRefreshEnable-0#:DisabledDuringSleep/1:Enabled */
	#define	CFG_SDRAM_CFG_ECC_EN		0	/* ECC_Enable-0#:Disable/1:Enable */
	#define	CFG_SDRAM_CFG_RD_EN			0	/* RegisteredDIMM-0#:Unbuffered/1:Buffered */
	#define	CFG_SDRAM_CFG_SDRAM_TYPE	3	/* SDRAMtype-0-1:Rsv/2#:DDR/3:DDR2/4-7:Rsv */
	#define	CFG_SDRAM_CFG_DYN_PWR		0	/* DynamicPowerManagement-0#:Disable/1:Enable */
	#define	CFG_SDRAM_CFG_32_BE			0	/* 32bitBusEnable-0#:64bit/1:32bit */
	#define	CFG_SDRAM_CFG_8_BE			0	/* 8beatBurstEnable-0#:4beat/1:8beat */
	#define	CFG_SDRAM_CFG_NCAP			0	/* Non-concurrentAuto-precharge-0#:SupportConcurrent/1:NOT-support */
	#define	CFG_SDRAM_CFG_2T_EN			0	/* 2T-Timing-0#:1T/1:2T */
	#define	CFG_SDRAM_CFG_BA_INTLV_CTL	0	/* BankInteleaving-0#:NoInterleave/1-127:Interleaving(See RefManual for detail) */
	#define	CFG_SDRAM_CFG_x32_EN		0	/* x32Enable-0#:x8 or x16 DRAM/1:x32 DRAM chip */
	#define	CFG_SDRAM_CFG_PCHB8			0	/* PrechargeBit8Enable-0#:MA[10]/1:MA[8] */
	#define	CFG_SDRAM_CFG_HSE			0	/* GlobalHalf-StrengthOverride-0#:FullStrength/1:HalfStrength */
	#define	CFG_SDRAM_CFG_MEM_HALT		0	/* DDRcontrollerHalt-0#:NOP/1:Halt */
	#define	CFG_SDRAM_CFG_BI			0	/* BypassInitialization-0#:Normal/1:Bypass */
	#if defined(CONFIG_DDR_2T_TIMING)
		#undef	CFG_SDRAM_CFG_2T_EN
		#define	CFG_SDRAM_CFG_2T_EN		1
	#endif
	#define	CFG_SDRAM_CFG		MKCFG_SDRAM_CFG( )
	#define	CFG_DDR_CONTROL		CFG_SDRAM_CFG

	#define	CFG_SDRAM_CFG2_FRC_SR		0	/* ForceSelfRefresh-0#:Normal/1:ForceSelfRefresh */
	#define	CFG_SDRAM_CFG2_SR_IE		0	/* SelfRefreshInterruptEnable-0#:Disable/1:Enable */
	#define	CFG_SDRAM_CFG2_DLL_RST_DIS	0	/* DLLresetDisable-0#:ResetEnable/1:Disable */
	#define	CFG_SDRAM_CFG2_ODT_CFG		2	/* ODTconfig-0#:Never/1:ODTonlyDuringWr/2:OnlyDuringRead/3:Always */
	#define	CFG_SDRAM_CFG2_NUM_PR		1	/* NumberOfPostedRefresh-0#:Rsv/1:1refresh/2:2refresh/.../8:8refresh/9-15:Rsv */
	#define	CFG_SDRAM_CFG2_D_INIT		0	/* DRAMdataInit-0#:NOP/1:InitializeMemory */
	#define	CFG_SDRAM_CFG2		MKCFG_SDRAM_CFG2( )

	#define	CFG_DDR_MODE_QOFF		0	/* OutputBufferDisable-0:Enable/1:Disable */
	#define	CFG_DDR_MODE_RDQS		0	/* RDQSenable-0:Disable/1:Enable */
	#define	CFG_DDR_MODE_xDQS		1	/* xDQSdisable-0:Enable/1:Disable */
	#define	CFG_DDR_MODE_OCD_PROG	0	/* OCDcalibrationMode-0:Normal/1:Drive1/2:Drive0/3:Rsv/4:AdjustMode/5-6:Rsv/7:CalibrationDefault */
	#define	CFG_DDR_MODE_RTT		1	/* ODTimpedance-0:ODTdisable/1:70ohm/2:150ohm/3:50ohm */
	#define	CFG_DDR_MODE_AL			CFG_TIMING_CFG2_ADD_LAT	/* AdditiveLatency-0:0clk/1:1clk/.../4:4clk/5-7:Rsv */
	#define	CFG_DDR_MODE_DSC		0	/* OutputDriverStrength-0:Normal(100%)/1:weak(60%) */
	#define	CFG_DDR_MODE_DLL_DIS	0	/* DLLdisable-0:Enable/1:Disable */
	#define	CFG_DDR_MODE_PD_EXIT	0	/* ActivePowerDownExitTiming-0:Fast/1:Slow */
	#define	CFG_DDR_MODE_WR			(CFG_TIMING_CFG1_WRREC - 1 )	/* WriteRecoverySelection-0:Rsv/1:2clk/2:3clk/.../5:6clk/6-7:Rsv */
	#define	CFG_DDR_MODE_DLL_RST	0	/* DLLreset-0:No/1:Yes */
	#define	CFG_DDR_MODE_TM			0	/* TestMode-0:Normal/1:TEST */
	#define	CFG_DDR_MODE_CL			((CFG_TIMING_CFG1_CASLAT>>1) + 1)	/* CAS_Latency-0-2:Rsv/3:3clk/4:4clk/5:5clk/6-7:Rsv */
	#define	CFG_DDR_MODE_BT			0	/* BurstType-0:Sequential */
	#define	CFG_DDR_MODE_BL			2	/* BurstLength(2'sPower)-2:4burst/3:8burst/Others(0-7):Rsv */
	#define	CFG_SDRAM_MODE		MKCFG_SDRAM_MODE( )
	#define	CFG_DDR_MODE		CFG_SDRAM_MODE

	#define	CFG_SDRAM_INTERVAL_REFINT	0x0400	/* RefreshInterval-0-65535:IntervalLength */
	#define	CFG_SDRAM_INTERVAL_BSTOPRE	0		/* PrechargeInterval-0#:Auto-precharge/1-65535:IntervalLength */
	#define	CFG_SDRAM_INTERVAL	MKCFG_SDRAM_INTERVAL( )
	#define	CFG_DDR_INTERVAL	CFG_SDRAM_INTERVAL

	#define	CFG_SDRAM_CLK_CNTL_CLK_ADJ	4	/* ClockAdjust-0:0cyc/1:1o-cyc/2:2o-cyc/.../4#:4o-cyc/.../8:8o-cyc/9-15:Rsv (o=1/8 cycle) */
	#define	CFG_SDRAM_CLK_CNTL	MKCFG_SDRAM_CLK_CNTL( )
#else	/* ^- CONFIG_BELLINI == MP */
	#error	unsupported model code, set DDR parameters and regist before building project.
#endif	/* ^- CONFIG_BELLINI == default */
#endif	/* CONFIG_BELLINI */
#endif	/* CONFIG_SPD_EEPROM */

/*
 * FLASH on the Local Bus
 */
#define CFG_FLASH_CFI			/* use the Common Flash Interface */
#define CFG_FLASH_CFI_DRIVER			/* use the CFI driver */
#define CFG_FLASH_BASE		0xFE000000	/* start of FLASH   */
#define CFG_FLASH_SIZE_LOG2	25			/* 2^25=32MB */
#define	CFG_FLASH_SIZE	(1 << ( CFG_FLASH_SIZE_LOG2 - SIZE_1MB_LOG2 ))	/* SIZE in MB */
/* #define CFG_FLASH_USE_BUFFER_WRITE */

	/* Spansion S29GL256N */
#define CFG_MAX_FLASH_BANKS	1		/* number of banks */
#define CFG_MAX_FLASH_SECT	256		/* sectors per device */

#undef	CFG_FLASH_CHECKSUM
#define CFG_FLASH_ERASE_TOUT	60000	/* Flash Erase Timeout (ms) */
#define CFG_FLASH_WRITE_TOUT	500	/* Flash Write Timeout (ms) */

#define CFG_MID_FLASH_JUMP      0x7F000000
#define CFG_MONITOR_BASE    	TEXT_BASE	/* start of monitor */

#if (CFG_MONITOR_BASE < CFG_FLASH_BASE)
#define CFG_RAMBOOT
#else
#undef  CFG_RAMBOOT
#endif

/*
 *	ChipSelect setting on the local bus
 */
#define	CONFIG_SETUP_UPM	1	/* use setupUPM() function */

#if defined( CONFIG_SETUP_UPM )
	#define	CONFIG_USE_UPMA		1	/* If defined, it enables UPM-x, using with CONFIG_SETUP_UPM option */
	#define	CONFIG_USE_UPMB		1
	#define	CONFIG_USE_UPMC		1
/*
	Dummy access facilities: 
	Writing UPM-RAM needs to familiar local bus access (so-called dummy write) with setting MnMR[OP] register except 0(NormalOp).
	And no setting LAW and LBIU.BR/OR caused bus fault, so we have to map dummy access address to somewhere.
	Freescale says this causes no actual external signals assertion (e.g. LCS[]), therefore you can choice anywhere for dummy access address,
	though you should still avoid address executing setUPM() function.
	(Why not direct read/write access from/to UPM-RAM array via MAD is capable? >> Freescale)
*/
	#define	CFG_UPM_DUMMY_BA				0xC5000000		/* Base address for dummy access in programming UPM-RAM. */
	#define	CFG_UPM_DUMMY_LBLAW_BANK		3				/* Local Bus LAW bank number */
	#define	CFG_UPM_DUMMY_LBLAW_BA			CFG_UPM_DUMMY_BA
	#define	CFG_UPM_DUMMY_LBLAW_SIZE_LOG2	12				/* 2^12 = 4KB (Minimum of LBLAW. Though LBIU dummy bank size exceeds it, but no access will be taken to outside. */
	#define	CFG_UPM_DUMMY_BR_BANK			7				/* LBIU CS bank number */
	#define	CFG_UPM_DUMMY_BR_BA				CFG_UPM_DUMMY_BA
	#define	CFG_UPM_DUMMY_BR_SIZE			ORxU_AM_MSK;	/* Narrowest size assigned (32KB) */
#endif	/* CONFIG_SETUP_UPM */

#define CFG_BR0_BA			CFG_FLASH_BASE			/* BaseAddress for local bus 0 */
#define CFG_BR0_PS			BR_PS_16				/* PortSize-0(#):Rsv/1:8bit/2:16bit/3:32bit */
#define	CFG_BR0_SEL			GPCM					/* BindingControlMachine:GPCM/UPMA/UPMB/UPMC */
#define	CFG_OR0_SIZE_LOG2	CFG_FLASH_SIZE_LOG2		/* CS area size (basis of AddressMask) */
#define CFG_BR0_PRELIM	MKCFG_LBC_BR_DFL(0)
#define CFG_OR0_PRELIM	MKCFG_LBC_OR_DFL(0)

/* When you choice UPM for MSEL field in some BRs, you have to set also CONFIG_SETUP_UPM, CONFIG_USE_UPM* definitions. */
#define CFG_BR1_BA			0xCE000000	/* for 8bit devices (3.3/NA) */
#define CFG_BR1_PS			BR_PS_8
#define	CFG_BR1_SEL			UPMA
#define	CFG_OR1_SIZE_LOG2	25		/* 2^25=32MB */

#define CFG_BR2_BA			0xC7000000	/* for 32bit devices (3.3/SA) */
#define CFG_BR2_PS			BR_PS_32
#define CFG_BR2_SEL			UPMA
#define	CFG_OR2_SIZE_LOG2	24		/* 2^24=16MB */

#define CFG_BR3_BA			0xF0000000	/* for DOC (3.3/NA) */
#define CFG_BR3_PS			BR_PS_16
#define CFG_BR3_SEL			UPMA
#define	CFG_OR3_SIZE_LOG2	17		/* 2^17=128KB */

#define CFG_BR4_BA			0xC0000000	/* for 32bit devices (3.3/SA) */
#define CFG_BR4_PS			BR_PS_32
#define CFG_BR4_SEL			UPMB
#define	CFG_OR4_SIZE_LOG2	26		/* 2^26=64MB */

#define CFG_BR5_BA			0xC4000000	/* for 32bit devices (3.3/SB) */
#define CFG_BR5_PS			BR_PS_32
#define CFG_BR5_SEL			UPMB
#define	CFG_OR5_SIZE_LOG2	24		/* 2^24=16MB */

#define CFG_BR6_BA			0xC6000000	/* for 32bit devices (3.3/SB) */
#define CFG_BR6_PS			BR_PS_32
#define CFG_BR6_SEL			UPMA
#define	CFG_OR6_SIZE_LOG2	24		/* 2^24=16MB */

#define CFG_BR7_BA			0xC8000000	/* for 8bit devices (3.3/NA) */
#define CFG_BR7_PS			BR_PS_16
#define CFG_BR7_SEL			UPMC
#define	CFG_OR7_SIZE_LOG2	26		/* 2^26=64MB */

#define CFG_BR1_PRELIM	MKCFG_LBC_BR_DFL(1)
#define CFG_OR1_PRELIM	MKCFG_LBC_OR_DFL(1)
#define CFG_BR2_PRELIM	MKCFG_LBC_BR_DFL(2)
#define CFG_OR2_PRELIM	MKCFG_LBC_OR_DFL(2)
#define CFG_BR3_PRELIM	MKCFG_LBC_BR_DFL(3)
#define CFG_OR3_PRELIM	MKCFG_LBC_OR_DFL(3)
#define CFG_BR4_PRELIM	MKCFG_LBC_BR_DFL(4)
#define CFG_OR4_PRELIM	MKCFG_LBC_OR_DFL(4)
#define CFG_BR5_PRELIM	MKCFG_LBC_BR_DFL(5)
#define CFG_OR5_PRELIM	MKCFG_LBC_OR_DFL(5)
#define CFG_BR6_PRELIM	MKCFG_LBC_BR_DFL(6)
#define CFG_OR6_PRELIM	MKCFG_LBC_OR_DFL(6)
#define CFG_BR7_PRELIM	MKCFG_LBC_BR_DFL(7)
#define CFG_OR7_PRELIM	MKCFG_LBC_OR_DFL(7)


/* ---------------------------------------------------------- */
/*
LAW(Local Access Window) configuration:
entry
	start addr	end addr	Description		size (bytes)

0	0xFE00_0000	0xFFFF_FFFF	Flash			32M
1	0xC000_0000	0xCFFF_FFFF	I/O				256M
2	0xF000_0000	0xF003_FFFF	DOC				256K
3	(unused)
*/

#define CFG_LBLAWBAR0_PRELIM	CFG_FLASH_BASE		/* Window base at flash base */
#define	CFG_LBLAWAR0_SIZE_LOG2	CFG_FLASH_SIZE_LOG2

#define CFG_LBLAWBAR1_PRELIM	0xC0000000			/* Window base for on-board peripherals */
#define	CFG_LBLAWAR1_SIZE_LOG2	28					/* 2^28=256MB */

#define	CFG_DOC_BASE			0xF0000000
#define CFG_LBLAWBAR2_PRELIM	CFG_DOC_BASE		/* Window base at DOC */
#define	CFG_LBLAWAR2_SIZE_LOG2	18					/* 2^18=256KB */

#define	CFG_LBLAWAR0_PRELIM		MKCFG_LBLAWAR_DFL(0)
#define	CFG_LBLAWAR1_PRELIM		MKCFG_LBLAWAR_DFL(1)
#define	CFG_LBLAWAR2_PRELIM		MKCFG_LBLAWAR_DFL(2)


#define CONFIG_L1_INIT_RAM
#define CFG_INIT_RAM_LOCK 	1
#define CFG_INIT_RAM_ADDR	0xe4010000   /* Initial RAM address */
#define CFG_INIT_RAM_END    	0x1000	     /* End of used area in RAM*/

#define CFG_GBL_DATA_SIZE  	0x100     /* num bytes initial data */
#define CFG_GBL_DATA_OFFSET	(CFG_INIT_RAM_END - CFG_GBL_DATA_SIZE)
#define CFG_INIT_SP_OFFSET	CFG_GBL_DATA_OFFSET

#define CFG_MONITOR_LEN	    	(256 * 1024) /* Reserve 256 kB for Mon */
#define CFG_MALLOC_LEN	    	(128 * 1024) /* Reserved for malloc */

/*
 * Local Bus LCRR and LBCR regs
 *    LCRR:  DLL bypass, Clock divider is 4
 * External Local Bus rate is
 *    CLKIN * HRCWL_CSB_TO_CLKIN / HRCWL_LCL_BUS_TO_SCB_CLK / LCRR_CLKDIV
 */
#define CFG_LCRR	(LCRR_DBYP | LCRR_CLKDIV_4)
#define CFG_LBC_LBCR	0x00000000


/*
 * Serial Port
 */
#define CONFIG_CONS_INDEX     1
#undef	CONFIG_SERIAL_SOFTWARE_FIFO
#define CFG_NS16550
#define CFG_NS16550_SERIAL
#define CFG_NS16550_REG_SIZE    1
#define CFG_NS16550_CLK		get_bus_freq(0)

#define CFG_BAUDRATE_TABLE  \
	{300, 600, 1200, 2400, 4800, 9600, 19200, 38400,115200}

#define CFG_NS16550_COM1        (CFG_IMMRBAR+0x4500)
#define CFG_NS16550_COM2        (CFG_IMMRBAR+0x4600)

/* Use the HUSH parser */
#define CFG_HUSH_PARSER
#ifdef  CFG_HUSH_PARSER
#define CFG_PROMPT_HUSH_PS2 "> "
#endif
#define	CONFIG_AUTO_COMPLETE	1

/* I2C */
#define  CONFIG_HARD_I2C		/* I2C with hardware support*/
#undef	CONFIG_SOFT_I2C			/* I2C bit-banged */
#define CFG_I2C_SPEED		100000	/* I2C speed and slave address (ignored in MPC83xx) */
#define CFG_I2C_SLAVE		0x7F
#define CFG_I2C_NOPROBES        {0x69}	/* Don't probe these addrs */
#define CFG_I2C_OFFSET      0x3000
#define CFG_I2C2_OFFSET      0x3100
/* See also TSEC section for handling errata TSEC6.1, CFG_I2C*_FDR are defined there */

#define	CFG_I2C_EEPROM_ADDR_LEN	2
#define	CFG_I2C_EEPROM_ADDR		0x50

#define	CONFIG_RTC_RV5C387A	1

/* IDE support */
#undef CONFIG_IDE_RESET		/* reset for ide supported	*/
#undef CONFIG_IDE_PREINIT
#undef CONFIG_COMPACT_FLASH

#define CFG_IDE_MAXBUS		1	/* max. 1 IDE bus		*/
#define CFG_IDE_MAXDEVICE	1	/* max. 1 drive per IDE bus	*/

#define CFG_ATA_IDE0_OFFSET	0x0000
#define CFG_ATA_BASE_ADDR	0xf0000000
#define CFG_ATA_DATA_OFFSET	(0x0000)
#define CFG_ATA_STRIDE		2
#define CFG_ATA_REG_OFFSET	0x0000
#define CFG_ATA_ALT_OFFSET	0x0200

#define CONFIG_DOS_PARTITION

/* TSEC */
#define CFG_TSEC1_OFFSET 0x24000
#define CFG_TSEC1 (CFG_IMMRBAR+CFG_TSEC1_OFFSET)
#define CFG_TSEC2_OFFSET 0x25000
#define CFG_TSEC2 (CFG_IMMRBAR+CFG_TSEC2_OFFSET)

  /* Errata TSEC6.1 */
#if defined( CONFIG_PATCH_ERRATA_TSEC_6_1 )
	#define CFG_ACR_PIPE_DEP	3	/* Arbiter pipeline depth (0-3) */
/*	#define CFG_ACR_PCIRPTCNT	0 (CPU default)*/	/* Arbiter repeat count (0-7) */
/*	#define CFG_ACR_RPTCNT		3 (CPU default)*/	/* Arbiter repeat count (0-7) */
	#define CFG_SPCR_TSEC1BDP	1	/* TSEC1 Buffer Descriptor Priority (0-3) */
	#define CFG_SPCR_TSEC1EP	3	/* TSEC1 emergency priority (0-3) */
	#define CFG_SPCR_TSEC2BDP	1
	#define CFG_SPCR_TSEC2EP	3	/* TSEC2 emergency priority (0-3) */

	#define CFG_SCCR_TSEC1CM	1	/* TSEC1 clock mode (0-3) */
	#define CFG_SCCR_TSEC2CM	1	/* TSEC2 & I2C0 clock mode (0-3) */

	#define	CFG_I2C_FDR			0x0e		/* Somehow, I2C module shares Clock Divider with TSEC1. */
	#define	CFG_I2C2_FDR		CFG_I2C_FDR	/* So, these are also affected by errata patch */
#else	/* ^-  CONFIG_PATCH_ERRATA_TSEC_6_1 */
	#define	CFG_SCCR_TSEC1CM	2		/* TSEC1 ClockMode-0:Disable/1(#?):(1/1)/2:(1/2)/3#:(1/3) */
	#define	CFG_SCCR_TSEC2CM	2		/* TSEC1 ClockMode-0:Disable/1(#?):(1/1)/2:(1/2)/3#:(1/3) */
/*	#define	CFG_SCCR_ENCCM		3	*/	/* EncCore ClkMode-0:Disable/1:(1/1)/2:(1/2)/3#:(1/3) */
/*	#define	CFG_SCCR_USBMPHCM	3	*/	/* USB_MPH ClkMode-0:Disable/1:(1/1)/2:(1/2)/3#:(1/3) */
/*	#define	CFG_SCCR_USBDRCM	3	*/	/* USB_DR ClkMode-0:Disable/1:(1/1)/2:(1/2)/3#:(1/3) */
/*	#define	CFG_SCCR_PCICM		1	*/	/* PCI Complex ClkMode-0:Disable/1#:Enable */

	#define	CFG_I2C_FDR			0x0a	/* Clock Dividing factor */
	#define	CFG_I2C2_FDR		CFG_I2C_FDR
#endif	/* ^- !CONFIG_PATCH_ERRATA_TSEC_6_1 */

/* IO Configuration */
#define CFG_IO_CONF (\
	IO_CONF_UART |\
	IO_CONF_TSEC1 |\
	IO_CONF_IRQ0 |\
	IO_CONF_IRQ1 |\
	IO_CONF_IRQ2 |\
	IO_CONF_IRQ3 |\
	IO_CONF_IRQ4 |\
	IO_CONF_IRQ5 |\
	IO_CONF_IRQ6 |\
	IO_CONF_IRQ7 )


#if defined(CONFIG_TSEC_ENET)
#ifndef CONFIG_NET_MULTI
#define CONFIG_NET_MULTI 	1
#endif

#define CONFIG_GMII		1	/* MII PHY management */
#define CONFIG_MPC83XX_TSEC1	1
#define CONFIG_MPC83XX_TSEC1_NAME	"TSEC0"
#define CONFIG_MPC83XX_TSEC2	1
#define CONFIG_MPC83XX_TSEC2_NAME	"TSEC1"
#define TSEC1_PHY_ADDR		3
#define TSEC2_PHY_ADDR		5
#define TSEC1_PHYIDX		0
#define TSEC2_PHYIDX		0

#define	CONFIG_PHY_KS8721_CONFIG_PATCH	1	/* see drivers/tsec.c for detail */

/* Options are: TSEC[0-1] */
#define CONFIG_ETHPRIME		"TSEC0"

#endif	/* CONFIG_TSEC_ENET */

/*
 * Environment
 */
#ifndef CFG_RAMBOOT
	#define CFG_ENV_IS_IN_FLASH	1
	#define CFG_ENV_ADDR		(CFG_MONITOR_BASE + 0x40000)
	#define CFG_ENV_SECT_SIZE	0x40000	/* reserved 256K(two sector) for env */
	#define CFG_ENV_SIZE		0x2000	/* unit is byte, upto 0x40000 in the reserved(protected) area */
#else
	#define CFG_NO_FLASH		1	/* Flash is not usable now */
	#define CFG_ENV_IS_NOWHERE	1	/* Store ENV in memory only */
	#define CFG_ENV_ADDR		(CFG_MONITOR_BASE - 0x1000)
	#define CFG_ENV_SIZE		0x2000
#endif

#define CONFIG_LOADS_ECHO	1	/* echo on for serial download */
#define CFG_LOADS_BAUD_CHANGE	1	/* allow baudrate change */

/* Hash command enable switch */
#define  CONFIG_COMMANDS_DFL \
			(CONFIG_CMD_DFL	\
				| CFG_CMD_PING \
				| CFG_CMD_MII \
				| CFG_CMD_DHCP \
			)

#if defined(CFG_RAMBOOT)
	#define  CONFIG_COMMANDS_BOOTMODE_DFL \
				(CONFIG_COMMANDS_DFL \
					& ~( 0 \
						| CFG_CMD_ENV \
						| CFG_CMD_LOADS \
					) \
				)
#else	/* ^- CFG_RAMBOOT */
	/* Flash boot configuration */
	#define  CONFIG_COMMANDS_BOOTMODE_DFL \
				(CONFIG_COMMANDS_DFL \
				)
#endif	/* ^-!CFG_RAMBOOT */

#if defined(CONFIG_HARD_I2C) || defined(CONFIG_SOFT_I2C)
	#define	CONFIG_COMMANDS_BOOTMODE_I2C_DFL \
				(CONFIG_COMMANDS_BOOTMODE_DFL \
					| CFG_CMD_I2C \
					| CFG_CMD_DATE \
					| CFG_CMD_EEPROM \
				)
#else	/* ^- CFG_*_I2C */
	#define	CONFIG_COMMANDS_BOOTMODE_I2C_DFL \
				(CONFIG_COMMANDS_BOOTMODE_DFL \
				)
#endif	/* ^-!CFG_*_I2C */

#if defined(CFG_IDE_MAXBUS)
	#define	CONFIG_COMMANDS_BOOTMODE_I2C_IDE_DFL \
				(CONFIG_COMMANDS_BOOTMODE_I2C_DFL \
					| CFG_CMD_IDE \
					| CFG_CMD_FAT \
	/*				| CFG_CMD_REISER */ \
					| CFG_CMD_EXT2 \
				)
#else	/* ^- CFG_IDE_MAXBUS */
	#define	CONFIG_COMMANDS_BOOTMODE_I2C_IDE_DFL \
				(CONFIG_COMMANDS_BOOTMODE_I2C_DFL \
				)
#endif	/* ^-!CFG_IDE_MAXBUS */

#if defined(CONFIG_PCI)
	#define	CONFIG_COMMANDS_BOOTMODE_I2C_IDE_PCI_DFL \
				(CONFIG_COMMANDS_BOOTMODE_I2C_IDE_DFL \
						| CFG_CMD_PCI \
					)
#else	/* ^- CONFIG_PCI */
	#define CONFIG_COMMANDS_BOOTMODE_I2C_IDE_PCI_DFL \
				(CONFIG_COMMANDS_BOOTMODE_I2C_IDE_DFL \
				)
#endif	/* ^-!CONFIG_PCI */

#define  CONFIG_COMMANDS	CONFIG_COMMANDS_BOOTMODE_I2C_IDE_PCI_DFL

#include <cmd_confdefs.h>

#undef CONFIG_WATCHDOG			/* watchdog disabled */

/*
 * Miscellaneous configurable options
 */
#define CFG_LONGHELP			/* undef to save memory	*/
#define CFG_LOAD_ADDR	0x2000000	/* default load address */
#define CFG_PROMPT	"=> "		/* Monitor Command Prompt */

#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
	#define CFG_CBSIZE	1024		/* Console I/O Buffer Size */
#else
	#define CFG_CBSIZE	256		/* Console I/O Buffer Size */
#endif

#define CFG_PBSIZE (CFG_CBSIZE+sizeof(CFG_PROMPT)+16) /* Print Buffer Size */
#define CFG_MAXARGS	16		/* max number of command args */
#define CFG_BARGSIZE	CFG_CBSIZE	/* Boot Argument Buffer Size */
#define CFG_HZ		1000		/* decrementer freq: 1ms ticks */

/*
 * For booting Linux, the board info and command line data
 * have to be in the first 8 MB of memory, since this is
 * the maximum mapped by the Linux kernel during initialization.
 */
#define CFG_BOOTMAPSZ	(8 << 20)	/* Initial Memory map for Linux*/

/* Cache Configuration */
#define CFG_DCACHE_SIZE		32768
#define CFG_CACHELINE_SIZE	32
#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CFG_CACHELINE_SHIFT	5	/*log base 2 of the above value*/
#endif

#define CFG_RCWH_PCIHOST 0x80000000 /* PCIHOST  */

#define CFG_HRCW_LOW (\
	HRCWL_LCL_BUS_TO_SCB_CLK_1X1 |\
	HRCWL_DDR_TO_SCB_CLK_1X1 |\
	HRCWL_CSB_TO_CLKIN_8X1 |\
	HRCWL_VCO_1X2 |\
	HRCWL_CORE_TO_CSB_2X1)

#if defined(PCI_64BIT)
#define CFG_HRCW_HIGH (\
	HRCWH_PCI_HOST |\
	HRCWH_64_BIT_PCI |\
	HRCWH_PCI1_ARBITER_ENABLE |\
	HRCWH_PCI2_ARBITER_DISABLE |\
	HRCWH_CORE_ENABLE |\
	HRCWH_FROM_0X00000100 |\
	HRCWH_BOOTSEQ_DISABLE |\
	HRCWH_SW_WATCHDOG_DISABLE |\
	HRCWH_ROM_LOC_LOCAL_16BIT |\
	HRCWH_TSEC1M_IN_GMII |\
	HRCWH_TSEC2M_IN_GMII )
#else
#define CFG_HRCW_HIGH (\
	HRCWH_PCI_HOST |\
	HRCWH_32_BIT_PCI |\
	HRCWH_PCI1_ARBITER_ENABLE |\
	HRCWH_PCI2_ARBITER_DISABLE |\
	HRCWH_CORE_ENABLE |\
	HRCWH_FROM_0XFFF00100 |\
	HRCWH_BOOTSEQ_DISABLE |\
	HRCWH_SW_WATCHDOG_DISABLE |\
	HRCWH_ROM_LOC_LOCAL_16BIT |\
	HRCWH_TSEC1M_IN_GMII |\
	HRCWH_TSEC2M_IN_GMII |\
	HRCWH_LDPARITY_UNUSED )
#endif

#define CFG_HID0_INIT 0x000000000

#define CFG_HID0_FINAL CFG_HID0_INIT
/* #define CFG_HID0_FINAL		(\
	HID0_ENABLE_INSTRUCTION_CACHE |\
	HID0_ENABLE_M_BIT |\
	HID0_ENABLE_ADDRESS_BROADCAST ) */

#define CFG_HID2 0x000000000

/*
 * Internal Definitions
 *
 * Boot Flags
 */
#define BOOTFLAG_COLD	0x01	/* Normal Power-On: Boot from FLASH */
#define BOOTFLAG_WARM	0x02	/* Software reboot */

#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CONFIG_KGDB_BAUDRATE	230400	/* speed of kgdb serial port */
#define CONFIG_KGDB_SER_INDEX	2	/* which serial port to use */
#endif


/*
 * Environment Configuration
 */

#if defined(CONFIG_TSEC_ENET)
#define CONFIG_ETHADDR   ff:ff:ff:ff:ff:ff
#define CONFIG_HAS_ETH1
#define CONFIG_ETH1ADDR  ff:ff:ff:ff:ff:ff
#define	CFG_ENVNAME_ETH0	"ethaddr"
#define	CFG_ENVNAME_ETH1	"eth1addr"
#endif

#define CONFIG_HOSTNAME	 Bellini
#define CONFIG_ROOTPATH	 /nfsroot
#define CONFIG_BOOTFILE	 your.uImage

#if 0
	#define CONFIG_IPADDR    192.168.1.253
	#define CONFIG_SERVERIP  192.168.1.1
	#define CONFIG_GATEWAYIP 192.168.1.1
	#define CONFIG_NETMASK   255.255.255.0
#endif	/* Never */

#define CONFIG_LOADADDR  1000000 	/* default location for tftp and bootm */

#define CONFIG_BOOTDELAY 0	/* -1=disables auto-boot, 0=no wait */
#undef  CONFIG_BOOTARGS		/* the boot command will set bootargs */

#define	CONFIG_BOOTSELECTKEY	1	/* enables boot select key (from stdin) feature */
#define	CFG_BOOTSELKEY_EMERGENCY	CTRL_CODE( 'E' )
#define	CFG_BOOTSELKEY_MAIN1	CTRL_CODE( 'A' )
#define	CFG_BOOTSELKEY_MAIN2	CTRL_CODE( 'B' )
#define	CFG_BOOTSELKEY_NOR		CTRL_CODE( 'D' )
#define	CFG_BOOTSELKEY_MON		CTRL_CODE( 'M' )
#if defined( CONFIG_BOOTSELECTKEY )
	#define	CONFIG_MENUPROMPT	""	/* supress "Hit any key..." message */
	#undef	CONFIG_AUTOBOOT_PROMPT
#else	/* ^- CONFIG_BOOTSELECTKEY */
	#define	CONFIG_ZERO_BOOTDELAY_CHECK
#endif	/* ^-!CONFIG_BOOTSELECTKEY */

#define CONFIG_BAUDRATE	 115200

/* Some environment variables are defined in board/Bellini/Bellini_local.h */
#define	CONFIG_EXTRA_ENV_SETTINGS \
	"netdev=eth0\0"					\
	"consoledev=ttyS0\0"				\
	""

#define CONFIG_BOOTCOMMAND "run boot0"

/* Definitions for the target(Bellini) internal use */
#if !defined( BV_FALSE )	/* Boolean value FALSE */
	#define	BV_FALSE	0
#endif	/* !BV_FALSE */
#if !defined( BV_TRUE )
	#define	BV_TRUE	1
#endif	/* !BV_TRUE */

/* Macros for the target */
#define	BITOP_SET( orig, data )		( (orig) | ( data) )
	/* BITOP_SET():     0(bit in data)=NO-Op(not operated), 1=set 1 to the corresponding bit in orig */
#define	BITOP_CLEAR( orig, data )	( (orig) & (~data) )
	/* BITOP_CLEAR():   0(bit in data)=NO-Op              , 1=set 0 to the corresponding bit in orig
	 * Take care of the bit value in 'data', it's reversed for the 'orig' value.
	 */
#define	BITOP_INSERT( orig, mask, data )	BITOP_SET( BITOP_CLEAR( orig, mask ), data )
	/* BITOP_INSERT():  0(bit in mask)=NO-Op(not operated), 1=copy bits in data to the corresponding bit in orig */

#define	EXCEPTION_IF( cond, label ) \
	if( cond ) \
	{ \
		goto label; \
	} \
  /* EXCEPTION_IF() */
#define	DEFENV( varname, val ) \
	if( getenv( varname ) == NULL ) \
	{ \
		setenv( varname, val ); \
	} \
  /* DEFENV() */
#define	ROUND_UP( val, base )	( (( ((val)-1) / (base) )+1) * (base) )	/* rounded up to multiple of 'base' */

#endif	/* __CONFIG_H */
