/*
 * Copyright (C) Sony corporation. 2007. All rights reserved.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 * Maintainer: AKA <masateru.akachi@jp.sony.com>
 *
 * Change log:
 * 20080526:	AKA <masateru.akachi@jp.sony.com>
 *          	*release
 */

#ifndef __CONFIG_BELLINI_MKCONST_H
#define __CONFIG_BELLINI_MKCONST_H

// unconfigurable parameters/macros
// shift quantities are also defined in asm-ppc/immap_83xx.h and depend on them.

#define	SIZE_1MB_LOG2		20	/* 2^20=1MB */

/*
 *	Local Access Window setting
 */
#define	MKCFG_LBLAWAR_DFL( bank )	MKCFG_LBLAWAR_DEF( _DFL, bank )	/* Generate Default setting value */
#define	MKCFG_LBLAWAR( bank )		MKCFG_LBLAWAR_DEF( bank, bank )
#define	MKCFG_LBLAWAR_UNUSED( )		(0 << LBLAWAR_EN_SHIFT)

#define	MKCFG_LBLAWAR_DEF( dfl_switch, bank ) \
			( 0 /* Don't refer directly in your C code */ \
				| (CFG_LBLAWAR##dfl_switch##_EN << LBLAWAR_EN_SHIFT) \
				| (( CFG_LBLAWAR##bank##_SIZE_LOG2 - 1 ) << LBLAWAR_SIZE_SHIFT) \
			)
#define	CFG_LBLAWAR_DFL_EN		1	/* WindowEnable-0#:Disable/1(#):Enable */
#define	LBLAWAR_EN_SHIFT		31
#define	LBLAWAR_SIZE_SHIFT		0

/*
 *	Local Bus Interface setting
 */

#define	MKCFG_LBC_BR_DFL( bank )				MKCFG_LBC_BR_DEF( _DFL, bank )
#define	MKCFG_LBC_BR( bank )					MKCFG_LBC_BR_DEF( bank, bank )

#define	MKCFG_LBC_BR_MSEL( machine )			(BR_MSEL_##machine)
#define	MKCFG_LBC_BR_DEF( dfl_switch, bank )	MKCFG_LBC_BR_DEF_( dfl_switch, bank, CFG_BR##bank##_SEL)
#define	MKCFG_LBC_BR_DEF_( dfl_switch, bank, machine ) \
			( 0	/* BR_BA, BR_*_SHIFT are defined in include/mpc83xx.h */ \
				| (CFG_BR##bank##_BA & BR_BA) \
				| (CFG_BR##bank##_PS << BR_PS_SHIFT) \
				| (MKCFG_LBC_BR_MSEL(machine) << BR_MSEL_SHIFT) \
				| (CFG_BR##dfl_switch##_V << BR_V_SHIFT) \
			)
#define	BR_PS_8			1
#define	BR_PS_16		2
#define	BR_PS_32		3
#define	BR_MSEL_GPCM	0
#define	BR_MSEL_SDRAM	3
#define	BR_MSEL_UPMA	4
#define	BR_MSEL_UPMB	5
#define	BR_MSEL_UPMC	6
#define	BR_VALID_YES	1
#define	BR_VALID_NO		0
#define CFG_BR_DFL_V	BR_VALID_YES	/* Valid-0(#):Invalid/1:Valid */


#define	MKCFG_LBC_OR_DFL( bank )						MKCFG_LBC_OR_DEF_( _DFL, bank, CFG_BR##bank##_SEL )
#define	MKCFG_LBC_OR( bank )							MKCFG_LBC_OR_DEF_( bank, bank, CFG_BR##bank##_SEL )

#define	MKCFG_LBC_OR_DEF_( dfl_switch, bank, machine )	MKCFG_LBC_OR_DEF( dfl_switch, bank, machine )	/* for expanding arg-"machine" */
#define	MKCFG_LBC_OR_DEF( dfl_switch, bank, machine )	MKCFG_LBC_OR_##machine##_DEF( dfl_switch, bank )
#define	ORxU_AM_MSK		0xFFFF8000
#define	MKCFG_LBC_OR_GPCM_DFL( bank )	MKCFG_LBC_OR_GPCM_DEF( _DFL, bank )
#define	MKCFG_LBC_OR_GPCM( bank )		MKCFG_LBC_OR_GPCM_DEF( bank, bank )
#define	MKCFG_LBC_OR_GPCM_DEF( dfl_switch, bank ) \
			( 0	/* This macro cannot support 4GB CS area :-) */ \
				| (( (( (1<<CFG_OR##bank##_SIZE_LOG2) - 1 ) ^ OR_GPCM_AM) & OR_GPCM_AM )) \
				| (CFG_OR##dfl_switch##_GPCM_BCTLD << OR_GPCM_BCTLD_SHIFT ) \
				| (CFG_OR##dfl_switch##_GPCM_CSNT << OR_GPCM_CSNT_SHIFT ) \
				| (CFG_OR##dfl_switch##_GPCM_ACS << OR_GPCM_ACS_SHIFT ) \
				| (CFG_OR##dfl_switch##_GPCM_XACS << OR_GPCM_XACS_SHIFT ) \
				| (CFG_OR##dfl_switch##_GPCM_SCY << OR_GPCM_SCY_SHIFT ) \
				| (CFG_OR##dfl_switch##_GPCM_SETA << OR_GPCM_SETA_SHIFT ) \
				| (CFG_OR##dfl_switch##_GPCM_TRLX << OR_GPCM_TRLX_SHIFT ) \
				| (CFG_OR##dfl_switch##_GPCM_EHTR << OR_GPCM_EHTR_SHIFT ) \
				| (CFG_OR##dfl_switch##_GPCM_EAD << OR_GPCM_EAD_SHIFT ) \
			)
#define	CFG_OR_DFL_GPCM_BCTLD	0				/* LBCTL_Control-0#:Enable/1:Disable */
#define	CFG_OR_DFL_GPCM_CSNT	1				/* CS_NegationTime-0(#):Normal/1:NegateEarlier */
#define	CFG_OR_DFL_GPCM_ACS		2				/* AddrToCS_Setup-0(#):SameTime/1:Rsv/2:delayedHalfClk/3:delayedQuarter(orHalf) */
#define	CFG_OR_DFL_GPCM_XACS	0				/* ExtraAddrToCS_Setup-0(#):Normal/1:Extended */
#define	CFG_OR_DFL_GPCM_SCY		5				/* CycleLength-0(#):NoWait/1:1clk/2:2clk/.../15:15clk */
#define	CFG_OR_DFL_GPCM_SETA	0				/* ExternalTermination-0(#):Ignore/1:xLGTAcanTerminateEarlier */
#define	CFG_OR_DFL_GPCM_TRLX	1				/* TimingRelaxed-0(#):Normal/1:Relaxed */
#define	CFG_OR_DFL_GPCM_EHTR	0				/* ExtendedHoldTime-0(#):Normal/1:Extended */
#define	CFG_OR_DFL_GPCM_EAD		1				/* ExternalAddrLatchDelay-0(#):NoAdditional/1:ExtraCycleAdded */
#define	MKCFG_LBC_OR_UPM_DFL( bank )	MKCFG_LBC_OR_UPM_DEF( _DFL, bank )
#define	MKCFG_LBC_OR_UPM( bank )		MKCFG_LBC_OR_UPM_DEF( bank, bank )
#define	MKCFG_LBC_OR_UPMA_DEF( dfl_switch, bank )	MKCFG_LBC_OR_UPM_DEF( dfl_switch, bank )
#define	MKCFG_LBC_OR_UPMB_DEF( dfl_switch, bank )	MKCFG_LBC_OR_UPM_DEF( dfl_switch, bank )
#define	MKCFG_LBC_OR_UPMC_DEF( dfl_switch, bank )	MKCFG_LBC_OR_UPM_DEF( dfl_switch, bank )
#define	MKCFG_LBC_OR_UPM_DEF( dfl_switch, bank ) \
			( 0	/* This macro cannot support 4GB CS area :-) */ \
				| (( (( (1<<CFG_OR##bank##_SIZE_LOG2) - 1 ) ^ OR_UPM_AM) & OR_UPM_AM )) \
				| (CFG_OR##dfl_switch##_UPM_BCTLD << OR_UPM_BCTLD_SHIFT ) \
				| (CFG_OR##dfl_switch##_UPM_BI << OR_UPM_BI_SHIFT ) \
				| (CFG_OR##dfl_switch##_UPM_TRLX << OR_UPM_TRLX_SHIFT ) \
				| (CFG_OR##dfl_switch##_UPM_EHTR << OR_UPM_EHTR_SHIFT ) \
				| (CFG_OR##dfl_switch##_UPM_EAD << OR_UPM_EAD_SHIFT ) \
			)
#define	CFG_OR_DFL_UPM_BCTLD	0	/* LBCTL_Control-0#:Enable/1:Disable */
#define	CFG_OR_DFL_UPM_BI		1	/* BurstInhibit-0#:CapableBurst/1:BurstInhibit */
#define	CFG_OR_DFL_UPM_TRLX		1	/* TimingRelaxed-0(#):Normal/1:Relaxed */
#define	CFG_OR_DFL_UPM_EHTR		0	/* ExtendedHoldTime-0(#):Normal/1:Extended */
#define	CFG_OR_DFL_UPM_EAD		1	/* ExternalAddrLatchDelay-0(#):NoAdditional/1:ExtraCycleAdded */


/*
 *	DDR Setting
 */
	/* DDRLAWBAR is not using any functional macro, described in <target>.c */
#define	MKCFG_DDRLAWAR_DFL( bank )	MKCFG_DDRLAWAR_DEF( _DFL, bank )
#define	MKCFG_DDRLAWAR( bank )		MKCFG_DDRLAWAR_DEF( bank, bank )
#define	MKCFG_DDRLAWAR_UNUSED( )	(0 << DDRLAWAR_EN_SHIFT)

#define	MKCFG_DDRLAWAR_DEF( dfl_switch, bank ) \
			( 0 \
				| (CFG_DDRLAWAR##dfl_switch##_EN << DDRLAWAR_EN_SHIFT) \
				| (( CFG_DDRLAW##bank##_SIZE_LOG2 - 1 ) << DDRLAWAR_SIZE_SHIFT) \
			)
#define	DDRLAWAR_EN_SHIFT		31
#define	CFG_DDRLAWAR_DFL_EN		1
#define	DDRLAWAR_SIZE_SHIFT		0


#define	MKCFG_CSBNDS( bank ) \
			( 0 \
				| (CSBNDS##bank##_SA << CSBNDS_SA_SHIFT) \
				| (CSBNDS##bank##_EA << CSBNDS_EA_SHIFT) \
			)

#define	MKCFG_CSCONFIG_UNUSED( )	(0 << CSCONFIG_EN_SHIFT)
#define	MKCFG_CSCONFIG( bank ) \
			( 0 \
				| (CFG_CSCONFIG##bank##_EN << CSCONFIG_EN_SHIFT) \
				| (CFG_CSCONFIG##bank##_AP << CSCONFIG_EN_SHIFT) \
				| (CFG_CSCONFIG##bank##_ODT_RD << CSCONFIG_ODT_RD_SHIFT) \
				| (CFG_CSCONFIG##bank##_ODT_WR << CSCONFIG_ODT_WR_SHIFT) \
				| (CFG_CSCONFIG##bank##_BA_BITS << CSCONFIG_BA_BITS_SHIFT) \
				| (CFG_CSCONFIG##bank##_ROW_BITS << CSCONFIG_ROW_BITS_SHIFT) \
				| (CFG_CSCONFIG##bank##_COL_BITS << CSCONFIG_COL_BITS_SHIFT) \
			)

#define	MKCFG_DDR_TIMING_0( ) \
			( 0 \
				| (CFG_TIMING_CFG0_RWT << TIMING_CFG0_RWT_SHIFT) \
				| (CFG_TIMING_CFG0_WRT << TIMING_CFG0_WRT_SHIFT) \
				| (CFG_TIMING_CFG0_RRT << TIMING_CFG0_RRT_SHIFT) \
				| (CFG_TIMING_CFG0_WWT << TIMING_CFG0_WWT_SHIFT) \
				| (CFG_TIMING_CFG0_ACT_PD_EXIT << TIMING_CFG0_ACT_PD_EXIT_SHIFT) \
				| (CFG_TIMING_CFG0_PRE_PD_EXIT << TIMING_CFG0_PRE_PD_EXIT_SHIFT) \
				| (CFG_TIMING_CFG0_ODT_PD_EXIT << TIMING_CFG0_ODT_PD_EXIT_SHIFT) \
				| (CFG_TIMING_CFG0_MRS_CYC << TIMING_CFG0_MRS_CYC_SHIFT) \
			)

#define	MKCFG_DDR_TIMING_1( ) \
			( 0 \
					| (CFG_TIMING_CFG1_PRETOACT << TIMING_CFG1_PRETOACT_SHIFT) \
					| (CFG_TIMING_CFG1_ACTTOPRE << TIMING_CFG1_ACTTOPRE_SHIFT) \
					| (CFG_TIMING_CFG1_ACTTORW << TIMING_CFG1_ACTTORW_SHIFT) \
					| (CFG_TIMING_CFG1_CASLAT << TIMING_CFG1_CASLAT_SHIFT) \
					| (CFG_TIMING_CFG1_REFREC << TIMING_CFG1_REFREC_SHIFT) \
					| (CFG_TIMING_CFG1_WRREC << TIMING_CFG1_WRREC_SHIFT) \
					| (CFG_TIMING_CFG1_ACTTOACT << TIMING_CFG1_ACTTOACT_SHIFT) \
					| (CFG_TIMING_CFG1_WRTORD << TIMING_CFG1_WRTORD_SHIFT) \
			)

#define MKCFG_DDR_TIMING_2( ) \
			( 0 \
				| (CFG_TIMING_CFG2_ADD_LAT << TIMING_CFG2_ADD_LAT_SHIFT) \
				| (CFG_TIMING_CFG2_CPO << TIMING_CFG2_CPO_SHIFT) \
				| (CFG_TIMING_CFG2_WR_LAT << TIMING_CFG2_WR_LAT_SHIFT) \
				| (CFG_TIMING_CFG2_RD_TO_PRE << TIMING_CFG2_RD_TO_PRE_SHIFT) \
				| (CFG_TIMING_CFG2_WR_DATA_DELAY << TIMING_CFG2_WR_DATA_DELAY_SHIFT) \
				| (CFG_TIMING_CFG2_CKE_PLS << TIMING_CFG2_CKE_PLS_SHIFT) \
				| (CFG_TIMING_CFG2_4_ACT << TIMING_CFG2_4_ACT_SHIFT) \
			)

#define	MKCFG_DDR_TIMING_3( ) \
			( 0 \
				| (CFG_TIMING_CFG3_EXT_REFREC << TIMING_CFG3_EXT_REFREC_SHIFT) \
			)

#define	MKCFG_SDRAM_CFG( ) \
			( 0 \
				| (CFG_SDRAM_CFG_SREN << SDRAM_CFG_SREN_SHIFT) \
				| (CFG_SDRAM_CFG_ECC_EN << SDRAM_CFG_ECC_EN_SHIFT) \
				| (CFG_SDRAM_CFG_RD_EN << SDRAM_CFG_RD_EN_SHIFT) \
				| (CFG_SDRAM_CFG_SDRAM_TYPE << SDRAM_CFG_SDRAM_TYPE_SHIFT) \
				| (CFG_SDRAM_CFG_DYN_PWR << SDRAM_CFG_DYN_PWR_SHIFT) \
				| (CFG_SDRAM_CFG_32_BE << SDRAM_CFG_32_BE_SHIFT) \
				| (CFG_SDRAM_CFG_8_BE << SDRAM_CFG_8_BE_SHIFT) \
				| (CFG_SDRAM_CFG_NCAP << SDRAM_CFG_NCAP_SHIFT) \
				| (CFG_SDRAM_CFG_2T_EN << SDRAM_CFG_2T_EN_SHIFT) \
				| (CFG_SDRAM_CFG_BA_INTLV_CTL << SDRAM_CFG_BA_INTLV_CTL_SHIFT) \
				| (CFG_SDRAM_CFG_x32_EN << SDRAM_CFG_x32_EN_SHIFT) \
				| (CFG_SDRAM_CFG_PCHB8 << SDRAM_CFG_PCHB8_SHIFT) \
				| (CFG_SDRAM_CFG_HSE << SDRAM_CFG_HSE_SHIFT) \
				| (CFG_SDRAM_CFG_MEM_HALT << SDRAM_CFG_MEM_HALT_SHIFT) \
				| (CFG_SDRAM_CFG_BI << SDRAM_CFG_BI_SHIFT) \
			)
#define	SDRAM_CFG_MEM_EN_SHIFT			31
#define	SDRAM_CFG_SREN_SHIFT			30
#define	SDRAM_CFG_ECC_EN_SHIFT			29
#define	SDRAM_CFG_RD_EN_SHIFT			28
#define	SDRAM_CFG_SDRAM_TYPE_SHIFT		24
#define	SDRAM_CFG_DYN_PWR_SHIFT			21
#define	SDRAM_CFG_32_BE_SHIFT			19
#define	SDRAM_CFG_8_BE_SHIFT			18
#define	SDRAM_CFG_NCAP_SHIFT			17
#define	SDRAM_CFG_2T_EN_SHIFT			15
#define	SDRAM_CFG_BA_INTLV_CTL_SHIFT	8
#define	SDRAM_CFG_x32_EN_SHIFT			5
#define	SDRAM_CFG_PCHB8_SHIFT			4
#define	SDRAM_CFG_HSE_SHIFT				3
#define	SDRAM_CFG_MEM_HALT_SHIFT		1
#define	SDRAM_CFG_BI_SHIFT				0

#define	MKCFG_SDRAM_CFG2( ) \
			( 0 \
				| (CFG_SDRAM_CFG2_FRC_SR << SDRAM_CFG2_FRC_SR_SHIFT) \
				| (CFG_SDRAM_CFG2_SR_IE << SDRAM_CFG2_SR_IE_SHIFT) \
				| (CFG_SDRAM_CFG2_DLL_RST_DIS << SDRAM_CFG2_DLL_RST_DIS_SHIFT) \
				| (CFG_SDRAM_CFG2_ODT_CFG << SDRAM_CFG2_ODT_CFG_SHIFT) \
				| (CFG_SDRAM_CFG2_NUM_PR << SDRAM_CFG2_NUM_PR_SHIFT) \
				| (CFG_SDRAM_CFG2_D_INIT << SDRAM_CFG2_D_INIT_SHIFT) \
			)
#define	MKCFG_SDRAM_MODE( ) \
			( 0 \
				| (0x1 << 30) | (0 << 29) \
				| (CFG_DDR_MODE_QOFF << DDR_MODE_QOFF_SHIFT) \
				| (CFG_DDR_MODE_RDQS << DDR_MODE_RDQS_SHIFT) \
				| (CFG_DDR_MODE_xDQS << DDR_MODE_xDQS_SHIFT) \
				| (CFG_DDR_MODE_OCD_PROG << DDR_MODE_OCD_PROG_SHIFT) \
				| (CFG_DDR_MODE_AL << DDR_MODE_AL_SHIFT) \
				| ((CFG_DDR_MODE_RTT >> 1) << DDR_MODE_RTT_HI_SHIFT) | ((CFG_DDR_MODE_RTT & 1) << DDR_MODE_RTT_LO_SHIFT) \
				| (CFG_DDR_MODE_DSC << DDR_MODE_DSC_SHIFT) \
				| (CFG_DDR_MODE_DLL_DIS << DDR_MODE_DLL_DIS_SHIFT) \
				| (0 << 14) | (0 << 13) \
				| (CFG_DDR_MODE_PD_EXIT << DDR_MODE_PD_EXIT_SHIFT) \
				| (CFG_DDR_MODE_WR << DDR_MODE_WR_SHIFT) \
				| (CFG_DDR_MODE_DLL_RST << DDR_MODE_DLL_RST_SHIFT) \
				| (CFG_DDR_MODE_TM << DDR_MODE_TM_SHIFT) \
				| (CFG_DDR_MODE_CL << DDR_MODE_CL_SHIFT) \
				| (CFG_DDR_MODE_BT << DDR_MODE_BT_SHIFT) \
				| (CFG_DDR_MODE_BL << DDR_MODE_BL_SHIFT) \
			)
#define	DDR_MODE_QOFF_SHIFT		28
#define	DDR_MODE_RDQS_SHIFT		27
#define	DDR_MODE_xDQS_SHIFT		26
#define	DDR_MODE_OCD_PROG_SHIFT	23
#define	DDR_MODE_RTT_HI_SHIFT	22
#define	DDR_MODE_AL_SHIFT		19
#define	DDR_MODE_RTT_LO_SHIFT	18
#define	DDR_MODE_DSC_SHIFT		17
#define	DDR_MODE_DLL_DIS_SHIFT	16
#define	DDR_MODE_PD_EXIT_SHIFT	12
#define	DDR_MODE_WR_SHIFT		9
#define	DDR_MODE_DLL_RST_SHIFT	8
#define	DDR_MODE_TM_SHIFT		7
#define	DDR_MODE_CL_SHIFT		4
#define	DDR_MODE_BT_SHIFT		3
#define	DDR_MODE_BL_SHIFT		0

#define	MKCFG_SDRAM_INTERVAL( ) \
			( 0 \
				| (CFG_SDRAM_INTERVAL_REFINT << SDRAM_INTERVAL_REFINT_SHIFT) \
				| (CFG_SDRAM_INTERVAL_BSTOPRE << SDRAM_INTERVAL_BSTOPRE_SHIFT) \
			)

#define	MKCFG_SDRAM_CLK_CNTL( ) \
			( 0 \
				| (CFG_SDRAM_CLK_CNTL_CLK_ADJ << SDRAM_CLK_CNTL_CLK_ADJ_SHIFT) \
			)

#endif	/* __CONFIG_BELLINI_MKCONST_H */
